@@ -41,11 +41,15 @@ if __name__ == '__main__':
 	gmDateTime.init()
 
 from Gnumed.pycommon import gmExceptions, gmPG2, gmDispatcher, gmI18N, gmCfg, gmTools, gmDateTime
+
 from Gnumed.business import gmAllergy
 from Gnumed.business import gmPathLab
 from Gnumed.business import gmClinNarrative
 from Gnumed.business import gmEMRStructItems
-from Gnumed.business import gmMedication, gmVaccination
+from Gnumed.business import gmMedication
+from Gnumed.business import gmVaccination
+from Gnumed.business import gmFamilyHistory
+from Gnumed.business.gmDemographicRecord import get_occupations
 
 
 _log = logging.getLogger('gm.emr')
@@ -147,6 +151,7 @@ SELECT fk_encounter from
 		return True
 	#--------------------------------------------------------
 	def db_callback_encounter_mod_db(self, **kwds):
+
 		# get the current encounter as an extra instance
 		# from the database to check for changes
 		curr_enc_in_db = gmEMRStructItems.cEncounter(aPK_obj = self.current_encounter['pk_encounter'])
@@ -196,6 +201,30 @@ SELECT fk_encounter from
 	def _clin_item_modified(self):
 		_log.debug('DB: clin_root_item modification')
 	#--------------------------------------------------------
+	# API: family history
+	#--------------------------------------------------------
+	def get_family_history(self, episodes=None, issues=None):
+		fhx = gmFamilyHistory.get_family_history (
+			order_by = u'l10n_relation, condition',
+			patient = self.pk_patient
+		)
+
+		if episodes is not None:
+			fhx = filter(lambda f: f['pk_episode'] in episodes, fhx)
+
+		if issues is not None:
+			fhx = filter(lambda f: f['pk_health_issue'] in issues, fhx)
+
+		return fhx
+	#--------------------------------------------------------
+	def add_family_history(self, episode=None, condition=None, relation=None):
+		return gmFamilyHistory.create_family_history (
+			encounter = self.current_encounter['pk_encounter'],
+			episode = episode,
+			condition = condition,
+			relation = relation
+		)
+	#--------------------------------------------------------
 	# API: performed procedures
 	#--------------------------------------------------------
 	def get_performed_procedures(self, episodes=None, issues=None):
@@ -673,23 +702,23 @@ order by
 	def get_statistics(self):
 		union_query = u'\n	union all\n'.join ([
 			u"""
-SELECT ((
-	-- all relevant health issues + active episodes WITH health issue
-	SELECT COUNT(1)
-	FROM clin.v_problem_list
-	WHERE
-		pk_patient = %(pat)s
-			AND
-		pk_health_issue is not null
-) + (
-	-- active episodes WITHOUT health issue
-	SELECT COUNT(1)
-	FROM clin.v_problem_list
-	WHERE
-		pk_patient = %(pat)s
-			AND
-		pk_health_issue is null
-))""",
+				SELECT ((
+					-- all relevant health issues + active episodes WITH health issue
+					SELECT COUNT(1)
+					FROM clin.v_problem_list
+					WHERE
+						pk_patient = %(pat)s
+							AND
+						pk_health_issue is not null
+				) + (
+					-- active episodes WITHOUT health issue
+					SELECT COUNT(1)
+					FROM clin.v_problem_list
+					WHERE
+						pk_patient = %(pat)s
+							AND
+						pk_health_issue is null
+				))""",
 			u'SELECT count(1) FROM clin.encounter WHERE fk_patient = %(pat)s',
 			u'SELECT count(1) FROM clin.v_pat_items WHERE pk_patient = %(pat)s',
 			u'SELECT count(1) FROM blobs.v_doc_med WHERE pk_patient = %(pat)s',
@@ -727,16 +756,17 @@ SELECT ((
 		return stats
 	#--------------------------------------------------------
 	def format_statistics(self):
-		return _("""Medical problems: %(problems)s
-Total encounters: %(encounters)s
-Total EMR entries: %(items)s
-Active medications: %(active_drugs)s
-Documents: %(documents)s
-Test results: %(results)s
-Hospital stays: %(stays)s
-Procedures: %(procedures)s
-Vaccinations: %(vaccinations)s
-"""			) % self.get_statistics()
+		return _(
+			'Medical problems: %(problems)s\n'
+			'Total encounters: %(encounters)s\n'
+			'Total EMR entries: %(items)s\n'
+			'Active medications: %(active_drugs)s\n'
+			'Documents: %(documents)s\n'
+			'Test results: %(results)s\n'
+			'Hospital stays: %(stays)s\n'
+			'Procedures: %(procedures)s\n'
+			'Vaccinations: %(vaccinations)s'
+		) % self.get_statistics()
 	#--------------------------------------------------------
 	def format_summary(self, dob=None):
 
@@ -757,6 +787,7 @@ Vaccinations: %(vaccinations)s
 				prob['problem'],
 				gmTools.bool2subst(prob['problem_active'], _('active'), _('inactive'))
 			)
+		txt += u'\n'
 		txt += _(' %s encounters from %s to %s\n') % (
 			stats['encounters'],
 			first['started'].strftime('%x').decode(gmI18N.get_encoding()),
@@ -769,7 +800,7 @@ Vaccinations: %(vaccinations)s
 		# FIXME: perhaps only count "ongoing ones"
 		txt += _(' %s performed procedures\n\n') % stats['procedures']
 
-		txt += _('Allergies and Intolerances\n\n')
+		txt += _('Allergies and Intolerances\n')
 
 		allg_state = self.allergy_state
 		txt += (u' ' + allg_state.state_string)
@@ -783,23 +814,43 @@ Vaccinations: %(vaccinations)s
 				gmTools.coalesce(allg['reaction'], _('unknown reaction'))
 			)
 
+		txt += u'\n'
+		txt += _('Family History')
+		txt += u'\n'
+		fhx = self.get_family_history()
+		for f in fhx:
+			txt += u'%s\n' % f.format(left_margin = 1)
+
+		txt += u'\n'
+		txt += _('Occupations')
+		txt += u'\n'
+		jobs = get_occupations(pk_identity = self.pk_patient)
+		for job in jobs:
+			txt += u' %s%s\n' % (
+				job['l10n_occupation'],
+				gmTools.coalesce(job['activities'], u'', u': %s')
+			)
+
 		txt += u'\n'
 		txt += _('Vaccinations')
 		txt += u'\n'
-
 		vaccs = self.get_latest_vaccinations()
 		inds = sorted(vaccs.keys())
 		for ind in inds:
 			ind_count, vacc = vaccs[ind]
-			txt += u' %s (%s%s): %s @ %s (%s %s%s%s)\n' % (
+			if dob is None:
+				age_given = u''
+			else:
+				age_given = u' @ %s' % gmDateTime.format_apparent_age_medically(gmDateTime.calculate_apparent_age (
+					start = dob,
+					end = vacc['date_given']
+				))
+			txt += u' %s (%s%s): %s%s (%s %s%s%s)\n' % (
 				ind,
 				gmTools.u_sum,
 				ind_count,
 				vacc['date_given'].strftime('%b %Y').decode(gmI18N.get_encoding()),
-				gmDateTime.format_apparent_age_medically(gmDateTime.calculate_apparent_age (
-					start = dob,
-					end = vacc['date_given']
-				)),
+				age_given,
 				vacc['vaccine'],
 				gmTools.u_left_double_angle_quote,
 				vacc['batch_no'],
@@ -808,7 +859,7 @@ Vaccinations: %(vaccinations)s
 
 		return txt
 	#--------------------------------------------------------
-	# allergy API
+	# API: allergy
 	#--------------------------------------------------------
  	def get_allergies(self, remove_sensitivities=False, since=None, until=None, encounters=None, episodes=None, issues=None, ID_list=None):
 		"""Retrieves patient allergy items.
@@ -949,7 +1000,7 @@ WHERE
 
 	allergy_state = property(_get_allergy_state, _set_allergy_state)
 	#--------------------------------------------------------
-	# episodes API
+	# API: episodes
 	#--------------------------------------------------------
 	def get_episodes(self, id_list=None, issues=None, open_status=None):
 		"""Fetches from backend patient episodes.
@@ -1066,7 +1117,7 @@ WHERE
 	def episode2problem(self, episode=None):
 		return gmEMRStructItems.episode2problem(episode=episode)
 	#--------------------------------------------------------
-	# problems API
+	# API: problems
 	#--------------------------------------------------------
 	def get_problems(self, episodes=None, issues=None, include_closed_episodes=False, include_irrelevant_issues=False):
 		"""Retrieve a patient's problems.
@@ -1145,7 +1196,7 @@ WHERE
 	def reclass_problem(self, problem):
 		return gmEMRStructItems.reclass_problem(problem = problem)
 	#--------------------------------------------------------
-	# health issues API
+	# API: health issues
 	#--------------------------------------------------------
 	def get_health_issues(self, id_list = None):
 
@@ -1223,7 +1274,13 @@ WHERE
 			preparation = preparation
 		)
 	#--------------------------------------------------------
-	# vaccinations API
+	def substance_intake_exists(self, pk_component=None):
+		return gmMedication.substance_intake_exists (
+			pk_component = pk_component,
+			pk_identity = self.pk_patient
+		)
+	#--------------------------------------------------------
+	# API: vaccinations
 	#--------------------------------------------------------
 	def add_vaccination(self, episode=None, vaccine=None, batch_no=None):
 		return gmVaccination.create_vaccination (
@@ -1286,14 +1343,14 @@ WHERE
 		if order_by is None:
 			order_by = u''
 		else:
-			order_by = u'order by %s' % order_by
+			order_by = u'ORDER BY %s' % order_by
 
 		if (episodes is not None) and (len(episodes) > 0):
 			where_parts.append(u'pk_episode IN %(epis)s')
 			args['epis'] = tuple(episodes)
 
 		if (issues is not None) and (len(issues) > 0):
-			where_parts.append(u'pk_episode IN (select pk from clin.episode where fk_health_issue IN %(issues)s)')
+			where_parts.append(u'pk_episode IN (SELECT pk FROM clin.episode WHERE fk_health_issue IN %(issues)s)')
 			args['issues'] = tuple(issues)
 
 		if (encounters is not None) and (len(encounters) > 0):
@@ -1915,37 +1972,17 @@ LIMIT 2
 			default = u'1 week'
 		)
 
-		# FIXME: this should be done async
-		cmd = u"""
-delete FROM clin.encounter
-WHERE
-	clin.encounter.fk_patient = %(pat)s
-		and
-	age(clin.encounter.last_affirmed) > %(ttl)s::interval
-		and
-	not exists (SELECT 1 FROM clin.clin_root_item WHERE fk_encounter = clin.encounter.pk)
-		and
-	not exists (SELECT 1 FROM blobs.doc_med WHERE fk_encounter = clin.encounter.pk)
-		and
-	not exists (SELECT 1 FROM clin.episode WHERE fk_encounter = clin.encounter.pk)
-		and
-	not exists (SELECT 1 FROM clin.health_issue WHERE fk_encounter = clin.encounter.pk)
-		and
-	not exists (SELECT 1 FROM clin.operation WHERE fk_encounter = clin.encounter.pk)
-		and
-	not exists (SELECT 1 FROM clin.allergy_state WHERE fk_encounter = clin.encounter.pk)
-"""
+#		# FIXME: this should be done async
+		cmd = u"select clin.remove_old_empty_encounters(%(pat)s::integer, %(ttl)s::interval)"
+		args = {'pat': self.pk_patient, 'ttl': ttl}
 		try:
-			rows, idx = gmPG2.run_rw_queries(queries = [{
-				'cmd': cmd,
-				'args': {'pat': self.pk_patient, 'ttl': ttl}
-			}])
+			rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 		except:
 			_log.exception('error deleting empty encounters')
 
 		return True
 	#------------------------------------------------------------------
-	# measurements API
+	# API: measurements
 	#------------------------------------------------------------------
 	# FIXME: use psyopg2 dbapi extension of named cursors - they are *server* side !
 	def get_test_types_for_results(self):
