@@ -1,7 +1,7 @@
 # -*- coding: utf8 -*-
 """Medication handling code.
 
-license: GPL
+license: GPL v2 or later
 """
 #============================================================
 __version__ = "$Revision: 1.21 $"
@@ -58,8 +58,8 @@ def drug2renal_insufficiency_url(search_term=None):
 	names = []
 
 	if isinstance(search_term, cBrandedDrug):
-		if search_term['atc_code'] is not None:
-			terms.append(search_term['atc_code'])
+		if search_term['atc'] is not None:
+			terms.append(search_term['atc'])
 
 	elif isinstance(search_term, cSubstanceIntakeEntry):
 		names.append(search_term['substance'])
@@ -81,7 +81,7 @@ def drug2renal_insufficiency_url(search_term=None):
 	#url_template = u'http://www.google.de/#q=site%%3Adosing.de+%s'
 	#url = url_template % u'+OR+'.join(terms)
 
-	url_template = u'http://www.google.de/search?hl=de&source=hp&q=site%%3Adosing.de+%s&btnG=Google-Suche'
+	url_template = u'http://www.google.com/search?hl=de&source=hp&q=site%%3Adosing.de+%s&btnG=Google-Suche'
 	url = url_template % u'+OR+'.join(terms)
 
 	_log.debug(u'renal insufficiency URL: %s', url)
@@ -671,10 +671,15 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 		atc_sens = [
 			a['atc_code'] for a in allgs if ((a['atc_code'] is not None) and (a['type'] == u'sensitivity'))
 		]
-		inn_allgs = [ a['allergene'] for a in allgs if a['type'] == u'allergy' ]
-		inn_sens = [ a['allergene'] for a in allgs if a['type'] == u'sensitivity' ]
+		inn_allgs = [
+			a['allergene'] for a in allgs if ((a['allergene'] is not None) and (a['type'] == u'allergy'))
+		]
+		inn_sens = [
+			a['allergene'] for a in allgs if ((a['allergene'] is not None) and (a['type'] == u'sensitivity'))
+		]
 		# this is rather fragile: FreeDiams won't know what type of UID this is
 		# (but it will assume it is of the type of the drug database in use)
+		# but eventually FreeDiams puts all drugs into one database :-)
 		uid_allgs = [
 			a['substance_code'] for a in allgs if ((a['substance_code'] is not None) and (a['type'] == u'allergy'))
 		]
@@ -748,18 +753,26 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 		prescription['ext_ref'] = u'FreeDiams'
 		prescription.save()
 		fd_filenames.append(filename)
-		success, msg, parts = prescription.add_parts_from_files (
-			files = fd_filenames,
-			reviewer = self.reviewer['pk_staff']
-		)
-
+		success, msg, parts = prescription.add_parts_from_files(files = fd_filenames)
 		if not success:
 			_log.error(msg)
 			return
 
+		for part in parts:
+			part['obj_comment'] = _('copy')
+			part.save()
+
 		xml_part = parts[-1]
 		xml_part['filename'] = u'freediams-prescription.xml'
+		xml_part['obj_comment'] = _('data')
 		xml_part.save()
+
+		# are we the intended reviewer ?
+		from Gnumed.business.gmPerson import gmCurrentProvider
+		me = gmCurrentProvider()
+		# if so: auto-sign the prescription
+		if xml_part['pk_intended_reviewer'] == me['pk_staff']:
+			prescription.set_reviewed(technically_abnormal = False, clinically_relevant = False)
 	#--------------------------------------------------------
 	def import_fd2gm_file_as_drugs(self, filename=None):
 		"""
@@ -1052,8 +1065,8 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 			new_drugs.append(drug)
 
 			# update fields
-			drug['is_fake'] = False
-			drug['atc_code'] = entry['atc']
+			drug['is_fake_brand'] = False
+			drug['atc'] = entry['atc']
 			drug['external_code_type'] = u'DE-PZN'
 			drug['external_code'] = entry['pzn']
 			drug['fk_data_source'] = data_src_pk
@@ -1297,7 +1310,7 @@ class cConsumableSubstance(gmBusinessDBObject.cBusinessDBObject):
 					WHERE
 						fk_drug_component IS NOT NULL
 							AND
-						fk_drug_component = (
+						fk_drug_component IN (
 							SELECT r_ls2b.pk
 							FROM ref.lnk_substance2brand r_ls2b
 							WHERE fk_substance = %(pk)s
@@ -1409,7 +1422,7 @@ class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 	_query1 = u"""
 		SELECT
 			pk::text,
-			(description || ' ' || amount || unit) as subst
+			(description || ' ' || amount || ' ' || unit) as subst
 		FROM ref.consumable_substance
 		WHERE description %(fragment_condition)s
 		ORDER BY subst
@@ -1417,7 +1430,7 @@ class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 	_query2 = u"""
 		SELECT
 			pk::text,
-			(description || ' ' || amount || unit) as subst
+			(description || ' ' || amount || ' ' || unit) as subst
 		FROM ref.consumable_substance
 		WHERE
 			%(fragment_condition)s
@@ -1666,6 +1679,22 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 		for test in tests:
 			print test.strip(), ":", regex.match(pattern, test.strip())
 #------------------------------------------------------------
+def substance_intake_exists(pk_component=None, pk_identity=None):
+	args = {'comp': pk_component, 'pat': pk_identity}
+	cmd = u"""
+SELECT exists (
+	SELECT 1 FROM clin.substance_intake
+	WHERE
+		fk_drug_component = %(comp)s
+			AND
+		fk_encounter IN (
+			SELECT pk FROM clin.encounter WHERE fk_patient = %(pat)s
+		)
+	LIMIT 1
+)"""
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+	return rows[0][0]
+#------------------------------------------------------------
 def create_substance_intake(pk_substance=None, pk_component=None, preparation=None, encounter=None, episode=None):
 
 	args = {
@@ -1729,13 +1758,13 @@ def format_substance_intake_notes(emr=None, output_format=u'latex', table_type=u
 	tex =  u'\n{\\small\n'
 	tex += u'\\noindent %s\n' % _('Additional notes')
 	tex += u'\n'
-	tex += u'\\noindent \\begin{tabular}{|l|l|l|l|}\n'
+	tex += u'\\noindent \\begin{tabularx}{\\textwidth}{|X|l|X|p{7.5cm}|}\n'
 	tex += u'\\hline\n'
-	tex += u'%s & %s & %s & \\\\ \n' % (_('Substance'), _('Strength'), _('Brand'))
+	tex += u'%s {\\scriptsize (%s)} & %s & %s \\\\ \n' % (_('Substance'), _('Brand'), _('Strength'), _('Advice'))
 	tex += u'\\hline\n'
 	tex += u'%s\n'
 	tex += u'\n'
-	tex += u'\\end{tabular}\n'
+	tex += u'\\end{tabularx}\n'
 	tex += u'}\n'
 
 	current_meds = emr.get_current_substance_intake (
@@ -1747,14 +1776,13 @@ def format_substance_intake_notes(emr=None, output_format=u'latex', table_type=u
 	# create lines
 	lines = []
 	for med in current_meds:
-
-		lines.append(u'%s & %s%s & %s %s & {\\scriptsize %s} \\\\ \n \\hline \n' % (
+		lines.append(u'%s%s %s & %s%s & %s \\\\ \n \\hline \n' % (
 			med['substance'],
+			gmTools.coalesce(med['brand'], u'', u' {\\scriptsize (%s)}'),
+			med['preparation'],
 			med['amount'],
 			med['unit'],
-			gmTools.coalesce(med['brand'], u''),
-			med['preparation'],
-			gmTools.coalesce(med['notes'], u'')
+			gmTools.coalesce(med['notes'], u'', u'{\\scriptsize %s}')
 		))
 
 	return tex % u' \n'.join(lines)
@@ -2022,7 +2050,7 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 	#--------------------------------------------------------
 	def set_substances_as_components(self, substances=None):
 
-		if self._payload[self._idx['is_in_use']]:
+		if self.is_in_use_by_patients:
 			return False
 
 		args = {'brand': self._payload[self._idx['pk_brand']]}
@@ -2148,6 +2176,28 @@ class cBrandedDrug(gmBusinessDBObject.cBusinessDBObject):
 		return rows[0][0]
 
 	is_vaccine = property(_get_is_vaccine, lambda x:x)
+	#--------------------------------------------------------
+	def _get_is_in_use_by_patients(self):
+		cmd = u"""
+			SELECT EXISTS (
+				SELECT 1
+				FROM clin.substance_intake
+				WHERE
+					fk_drug_component IS NOT NULL
+						AND
+					fk_drug_component IN (
+						SELECT r_ls2b.pk
+						FROM ref.lnk_substance2brand r_ls2b
+						WHERE fk_brand = %(pk)s
+					)
+				LIMIT 1
+			)"""
+		args = {'pk': self.pk_obj}
+
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		return rows[0][0]
+
+	is_in_use_by_patients = property(_get_is_in_use_by_patients, lambda x:x)
 #------------------------------------------------------------
 def get_branded_drugs():
 	cmd = u'SELECT pk FROM ref.branded_drug ORDER BY description'
@@ -2320,6 +2370,8 @@ if __name__ == "__main__":
 		for s in get_consumable_substances():
 			print s
 	#--------------------------------------------------------
+	def test_drug2renal_insufficiency_url():
+		drug2renal_insufficiency_url(search_term = 'Metoprolol')
 	#--------------------------------------------------------
 	# MMI/Gelbe Liste
 	#test_MMI_interface()
@@ -2330,7 +2382,7 @@ if __name__ == "__main__":
 	#test_mmi_import_drugs()
 
 	# FreeDiams
-	test_fd_switch_to()
+	#test_fd_switch_to()
 	#test_fd_show_interactions()
 
 	# generic
@@ -2338,4 +2390,6 @@ if __name__ == "__main__":
 	#test_create_substance_intake()
 	#test_show_components()
 	#test_get_consumable_substances()
+
+	test_drug2renal_insufficiency_url()
 #============================================================
