@@ -52,10 +52,16 @@ care of all the pre- and post-GUI runtime environment setup.
 #==========================================================
 __version__ = "$Revision: 1.169 $"
 __author__  = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
-__license__ = "GPL (details at http://www.gnu.org)"
+__license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 # standard library
-import sys, os, os.path, signal, logging, platform
+import sys
+import os
+import platform
+import logging
+import signal
+import os.path
+import shutil
 
 
 # do not run as module
@@ -79,8 +85,8 @@ against. Please run GNUmed as a non-root user.
 	sys.exit(1)
 
 #----------------------------------------------------------
-current_client_version = u'0.9.11'
-current_client_branch = u'0.9'
+current_client_version = u'1.1.7'
+current_client_branch = u'1.1'
 
 _log = None
 _cfg = None
@@ -441,7 +447,13 @@ def setup_ui_type():
 	_log.debug('UI type: %s', ui_type)
 #==========================================================
 def setup_backend():
-	_log.info('client expects database version [%s]', gmPG2.map_client_branch2required_db_version[current_client_branch])
+
+	db_version = gmPG2.map_client_branch2required_db_version[current_client_branch]
+	_log.info('client expects database version [%s]', db_version)
+	_cfg.set_option (
+		option = u'database_version',
+		value = db_version
+	)
 
 	# set up database connection timezone
 	timezone = _cfg.get (
@@ -488,7 +500,17 @@ def shutdown_logging():
 
 	# do not choke on Windows
 	logging.raiseExceptions = False
+#==========================================================
+def shutdown_tmp_dir():
 
+	tmp_dir = gmTools.gmPaths().tmp_dir
+
+	if _cfg.get(option = u'debug'):
+		_log.debug('not removing tmp dir (--debug mode): %s', tmp_dir)
+		return
+
+	_log.warning('removing tmp dir: %s', tmp_dir)
+	shutil.rmtree(tmp_dir, True)
 #==========================================================
 # main - launch the GNUmed wxPython GUI client
 #----------------------------------------------------------
@@ -536,6 +558,7 @@ elif ui_type == u'chweb':
 gmHooks.run_hook_script(hook = u'shutdown-post-GUI')
 
 shutdown_backend()
+shutdown_tmp_dir()
 _log.info('Normally shutting down as main module.')
 shutdown_logging()
 
