@@ -32,11 +32,15 @@ or imprecision of the timestamp contained within.
 
 This is useful in fields such as medicine where only partial
 timestamps may be known for certain events.
+
+Other useful links:
+
+	http://joda-time.sourceforge.net/key_instant.html
 """
 #===========================================================================
 __version__ = "$Revision: 1.34 $"
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
-__license__ = "GPL (details at http://www.gnu.org)"
+__license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 # stdlib
 import sys, datetime as pyDT, time, os, re as regex, locale, logging
@@ -249,7 +253,7 @@ def pydt_strftime(dt, format='%c', encoding=None, accuracy=None):
 		encoding = gmI18N.get_encoding()
 
 	try:
-		return dt.strftime(format).decode(encoding)
+		return dt.strftime(format).decode(encoding, 'replace')
 	except ValueError:
 		_log.exception('Python cannot strftime() this <datetime>')
 
@@ -332,7 +336,14 @@ def py_dt2wxDate(py_dt=None, wx=None):
 #===========================================================================
 # interval related
 #---------------------------------------------------------------------------
-def format_interval(interval=None, accuracy_wanted=acc_seconds):
+def format_interval(interval=None, accuracy_wanted=None, none_string=None):
+
+	if accuracy_wanted is None:
+		accuracy_wanted = acc_seconds
+
+	if interval is None:
+		if none_string is not None:
+			return none_string
 
 	years, days = divmod(interval.days, avg_days_per_gregorian_year)
 	months, days = divmod(days, avg_days_per_gregorian_month)
@@ -735,7 +746,8 @@ def __single_char2py_dt(str2parse, trigger_chars=None):
 
 	Spaces and tabs are discarded.
 
-	Default is 'dmy':
+	Default is 'ndmy':
+		n - Now
 		d - toDay
 		m - toMorrow	Someone please suggest a synonym !
 		y - Yesterday
@@ -743,7 +755,7 @@ def __single_char2py_dt(str2parse, trigger_chars=None):
 	This also defines the significance of the order of the characters.
 	"""
 	if trigger_chars is None:
-		trigger_chars = _('dmy (single character date triggers)')[:3].lower()
+		trigger_chars = _('ndmy (single character date triggers)')[:4].lower()
 
 	str2parse = str2parse.strip().lower()
 
@@ -756,15 +768,24 @@ def __single_char2py_dt(str2parse, trigger_chars=None):
 	now = mxDT.now()
 	enc = gmI18N.get_encoding()
 
-	# today
+	# FIXME: handle uebermorgen/vorgestern ?
+
+	# right now
 	if str2parse == trigger_chars[0]:
+		return [{
+			'data': mxdt2py_dt(now),
+			'label': _('right now (%s, %s)') % (now.strftime('%A').decode(enc), now)
+		}]
+
+	# today
+	if str2parse == trigger_chars[1]:
 		return [{
 			'data': mxdt2py_dt(now),
 			'label': _('today (%s)') % now.strftime('%A, %Y-%m-%d').decode(enc)
 		}]
 
 	# tomorrow
-	if str2parse == trigger_chars[1]:
+	if str2parse == trigger_chars[2]:
 		ts = now + mxDT.RelativeDateTime(days = +1)
 		return [{
 			'data': mxdt2py_dt(ts),
@@ -772,7 +793,7 @@ def __single_char2py_dt(str2parse, trigger_chars=None):
 		}]
 
 	# yesterday
-	if str2parse == trigger_chars[2]:
+	if str2parse == trigger_chars[3]:
 		ts = now + mxDT.RelativeDateTime(days = -1)
 		return [{
 			'data': mxdt2py_dt(ts),
@@ -1258,10 +1279,23 @@ def str2pydt_matches(str2parse=None, patterns=None):
 	if patterns is None:
 		patterns = []
 
-	patterns.append('%Y.%m.%d')
-	patterns.append('%Y/%m/%d')
 	patterns.append('%Y-%m-%d')
+	patterns.append('%y-%m-%d')
+	patterns.append('%Y/%m/%d')
+	patterns.append('%y/%m/%d')
+
+	patterns.append('%d-%m-%Y')
+	patterns.append('%d-%m-%y')
 	patterns.append('%d/%m/%Y')
+	patterns.append('%d/%m/%y')
+
+	patterns.append('%m-%d-%Y')
+	patterns.append('%m-%d-%y')
+	patterns.append('%m/%d/%Y')
+	patterns.append('%m/%d/%y')
+
+	patterns.append('%Y.%m.%d')
+	patterns.append('%y.%m.%d')
 
 	for pattern in patterns:
 		try:
@@ -1272,8 +1306,8 @@ def str2pydt_matches(str2parse=None, patterns=None):
 				tzinfo = gmCurrentLocalTimezone
 			)
 			matches.append ({
-				'data': mxdt2py_dt(date),
-				'label': date.strftime('%Y-%m-%d')
+				'data': date,
+				'label': pydt_strftime(date, format = '%Y-%m-%d', accuracy = acc_days)
 			})
 		except AttributeError:
 			# strptime() only available starting with Python 2.5
@@ -1287,7 +1321,7 @@ def str2pydt_matches(str2parse=None, patterns=None):
 
 	return matches
 #===========================================================================
-# string -> timestamp parser
+# string -> fuzzy timestamp parser
 #---------------------------------------------------------------------------
 def __explicit_offset(str2parse, offset_chars=None):
 	"""
@@ -1373,76 +1407,6 @@ def __explicit_offset(str2parse, offset_chars=None):
 	}
 	return [tmp]
 #---------------------------------------------------------------------------
-def __single_char(str2parse, trigger_chars=None):
-	"""This matches on single characters.
-
-	Spaces and tabs are discarded.
-
-	Default is 'ndmy':
-		n - now
-		d - toDay
-		m - toMorrow	Someone please suggest a synonym !
-		y - yesterday
-
-	This also defines the significance of the order of the characters.
-	"""
-	if trigger_chars is None:
-		trigger_chars = _('ndmy (single character date triggers)')[:4].lower()
-
-	if not regex.match(u'^(\s|\t)*[%s]{1}(\s|\t)*$' % trigger_chars, str2parse, flags = regex.LOCALE | regex.UNICODE):
-		return []
-	val = str2parse.strip().lower()
-
-	now = mxDT.now()
-	enc = gmI18N.get_encoding()
-
-	# FIXME: handle uebermorgen/vorgestern ?
-
-	# right now
-	if val == trigger_chars[0]:
-		ts = now
-		return [{
-			'data': cFuzzyTimestamp (
-				timestamp = ts,
-				accuracy = acc_subseconds
-			),
-			'label': _('right now (%s, %s)') % (ts.strftime('%A').decode(enc), ts)
-		}]
-
-	# today
-	if val == trigger_chars[1]:
-		return [{
-			'data': cFuzzyTimestamp (
-				timestamp = now,
-				accuracy = acc_days
-			),
-			'label': _('today (%s)') % now.strftime('%A, %Y-%m-%d').decode(enc)
-		}]
-
-	# tomorrow
-	if val == trigger_chars[2]:
-		ts = now + mxDT.RelativeDateTime(days = +1)
-		return [{
-			'data': cFuzzyTimestamp (
-				timestamp = ts,
-				accuracy = acc_days
-			),
-			'label': _('tomorrow (%s)') % ts.strftime('%A, %Y-%m-%d').decode(enc)
-		}]
-
-	# yesterday
-	if val == trigger_chars[3]:
-		ts = now + mxDT.RelativeDateTime(days = -1)
-		return [{
-			'data': cFuzzyTimestamp (
-				timestamp = ts,
-				accuracy = acc_days
-			),
-			'label': _('yesterday (%s)') % ts.strftime('%A, %Y-%m-%d').decode(enc)
-		}]
-
-	return []
-#---------------------------------------------------------------------------
 def __single_slash(str2parse):
 	"""Expand fragments containing a single slash.
 
@@ -1821,43 +1785,66 @@ def str2fuzzy_timestamp_matches(str2parse=None, default_time=None, patterns=None
 	matches = __single_dot(str2parse)
 	matches.extend(__numbers_only(str2parse))
 	matches.extend(__single_slash(str2parse))
-	matches.extend(__single_char(str2parse))
+	ms = __single_char2py_dt(str2parse)
+	for m in ms:
+		matches.append ({
+			'data': cFuzzyTimestamp (
+				timestamp = m['data'],
+				accuracy = acc_days
+			),
+			'label': m['label']
+		})
 	matches.extend(__explicit_offset(str2parse))
 
 	# try mxDT parsers
-	if mxDT is not None:
-		try:
-			# date ?
-			date_only = mxDT.Parser.DateFromString (
-				text = str2parse,
-				formats = ('euro', 'iso', 'us', 'altus', 'altiso', 'lit', 'altlit', 'eurlit')
-			)
-			# time, too ?
-			time_part = mxDT.Parser.TimeFromString(text = str2parse)
-			datetime = date_only + time_part
-			if datetime == date_only:
-				accuracy = acc_days
-				if isinstance(default_time, mxDT.DateTimeDeltaType):
-					datetime = date_only + default_time
-					accuracy = acc_minutes
-			else:
-				accuracy = acc_subseconds
-			fts = cFuzzyTimestamp (
-				timestamp = datetime,
-				accuracy = accuracy
-			)
-			matches.append ({
-				'data': fts,
-				'label': fts.format_accurately()
-			})
-		except (ValueError, mxDT.RangeError):
-			pass
+	try:
+		# date ?
+		date_only = mxDT.Parser.DateFromString (
+			text = str2parse,
+			formats = ('euro', 'iso', 'us', 'altus', 'altiso', 'lit', 'altlit', 'eurlit')
+		)
+		# time, too ?
+		time_part = mxDT.Parser.TimeFromString(text = str2parse)
+		datetime = date_only + time_part
+		if datetime == date_only:
+			accuracy = acc_days
+			if isinstance(default_time, mxDT.DateTimeDeltaType):
+				datetime = date_only + default_time
+				accuracy = acc_minutes
+		else:
+			accuracy = acc_subseconds
+		fts = cFuzzyTimestamp (
+			timestamp = datetime,
+			accuracy = accuracy
+		)
+		matches.append ({
+			'data': fts,
+			'label': fts.format_accurately()
+		})
+	except (ValueError, mxDT.RangeError):
+		pass
 
 	if patterns is None:
 		patterns = []
 
-	patterns.append(['%Y.%m.%d', acc_days])
+	patterns.append(['%Y-%m-%d', acc_days])
+	patterns.append(['%y-%m-%d', acc_days])
 	patterns.append(['%Y/%m/%d', acc_days])
+	patterns.append(['%y/%m/%d', acc_days])
+
+	patterns.append(['%d-%m-%Y', acc_days])
+	patterns.append(['%d-%m-%y', acc_days])
+	patterns.append(['%d/%m/%Y', acc_days])
+	patterns.append(['%d/%m/%y', acc_days])
+
+	patterns.append(['%m-%d-%Y', acc_days])
+	patterns.append(['%m-%d-%y', acc_days])
+	patterns.append(['%m/%d/%Y', acc_days])
+	patterns.append(['%m/%d/%y', acc_days])
+
+	patterns.append(['%Y.%m.%d', acc_days])
+	patterns.append(['%y.%m.%d', acc_days])
+
 
 	for pattern in patterns:
 		try:
@@ -1917,20 +1904,18 @@ class cFuzzyTimestamp:
 			accuracy = acc_subseconds
 			modifier = ''
 
+		if (accuracy < 1) or (accuracy > 8):
+			raise ValueError('%s.__init__(): <accuracy> must be between 1 and 8' % self.__class__.__name__)
+
 		if isinstance(timestamp, pyDT.datetime):
 			timestamp = mxDT.DateTime(timestamp.year, timestamp.month, timestamp.day, timestamp.hour, timestamp.minute, timestamp.second)
 
 		if type(timestamp) != mxDT.DateTimeType:
-			raise TypeError, '%s.__init__(): <timestamp> must be of mx.DateTime.DateTime or datetime.datetime type' % self.__class__.__name__
+			raise TypeError('%s.__init__(): <timestamp> must be of mx.DateTime.DateTime or datetime.datetime type' % self.__class__.__name__)
 
 		self.timestamp = timestamp
-
-		if (accuracy < 1) or (accuracy > 8):
-			raise ValueError, '%s.__init__(): <accuracy> must be between 1 and 7' % self.__class__.__name__
 		self.accuracy = accuracy
-
 		self.modifier =  modifier
-
 	#-----------------------------------------------------------------------
 	# magic API
 	#-----------------------------------------------------------------------
@@ -1960,34 +1945,37 @@ class cFuzzyTimestamp:
 	def Format(self, format_string):
 		return self.strftime(format_string)
 	#-----------------------------------------------------------------------
-	def format_accurately(self):
-		if self.accuracy == acc_years:
+	def format_accurately(self, accuracy=None):
+		if accuracy is None:
+			accuracy = self.accuracy
+
+		if accuracy == acc_years:
 			return unicode(self.timestamp.year)
 
-		if self.accuracy == acc_months:
+		if accuracy == acc_months:
 			return unicode(self.timestamp.strftime('%m/%Y'))	# FIXME: use 3-letter month ?
 
-		if self.accuracy == acc_weeks:
+		if accuracy == acc_weeks:
 			return unicode(self.timestamp.strftime('%m/%Y'))	# FIXME: use 3-letter month ?
 
-		if self.accuracy == acc_days:
+		if accuracy == acc_days:
 			return unicode(self.timestamp.strftime('%Y-%m-%d'))
 
-		if self.accuracy == acc_hours:
+		if accuracy == acc_hours:
 			return unicode(self.timestamp.strftime("%Y-%m-%d %I%p"))
 
-		if self.accuracy == acc_minutes:
+		if accuracy == acc_minutes:
 			return unicode(self.timestamp.strftime("%Y-%m-%d %H:%M"))
 
-		if self.accuracy == acc_seconds:
+		if accuracy == acc_seconds:
 			return unicode(self.timestamp.strftime("%Y-%m-%d %H:%M:%S"))
 
-		if self.accuracy == acc_subseconds:
+		if accuracy == acc_subseconds:
 			return unicode(self.timestamp)
 
 		raise ValueError, '%s.format_accurately(): <accuracy> (%s) must be between 1 and 7' % (
 			self.__class__.__name__,
-			self.accuracy
+			accuracy
 		)
 	#-----------------------------------------------------------------------
 	def get_mxdt(self):
@@ -2154,9 +2142,9 @@ if __name__ == '__main__':
 			matches = str2fuzzy_timestamp_matches(str2parse = val)
 			for match in matches:
 				print 'label shown  :', match['label']
-				print 'data attached:', match['data']
+				print 'data attached:', match['data'], match['data'].timestamp
 				print ""
-			print "---------------"	
+			print "---------------"
 	#-------------------------------------------------
 	def test_cFuzzyTimeStamp():
 		print "testing fuzzy timestamp class"
@@ -2246,7 +2234,7 @@ if __name__ == '__main__':
 	#test_format_interval()
 	#test_format_interval_medically()
 	#test_calculate_apparent_age()
-	#test_str2pydt()
-	test_pydt_strftime()
+	test_str2pydt()
+	#test_pydt_strftime()
 
 #===========================================================================
