@@ -4,7 +4,10 @@
 __version__ = "$Revision: 1.187 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import os.path, sys, re as regex, logging
+import os.path
+import sys
+import re as regex
+import logging
 
 
 import wx
@@ -13,9 +16,17 @@ import wx
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmI18N, gmCfg, gmPG2, gmMimeLib, gmExceptions, gmMatchProvider, gmDispatcher, gmDateTime, gmTools, gmShellAPI, gmHooks
-from Gnumed.business import gmPerson, gmDocuments, gmEMRStructItems, gmSurgery
-from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmPhraseWheel, gmPlugin, gmEMRStructWidgets, gmListWidgets
-from Gnumed.wxGladeWidgets import wxgReviewDocPartDlg, wxgSelectablySortedDocTreePnl
+from Gnumed.business import gmPerson
+from Gnumed.business import gmDocuments
+from Gnumed.business import gmEMRStructItems
+from Gnumed.business import gmSurgery
+
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmRegetMixin
+from Gnumed.wxpython import gmPhraseWheel
+from Gnumed.wxpython import gmPlugin
+from Gnumed.wxpython import gmEMRStructWidgets
+from Gnumed.wxpython import gmListWidgets
 
 
 _log = logging.getLogger('gm.ui')
@@ -95,9 +106,32 @@ def manage_document_descriptions(parent=None, document=None):
 	return True
 #============================================================
 def _save_file_as_new_document(**kwargs):
+	try:
+		del kwargs['signal']
+		del kwargs['sender']
+	except KeyError:
+		pass
 	wx.CallAfter(save_file_as_new_document, **kwargs)
+
+def _save_files_as_new_document(**kwargs):
+	try:
+		del kwargs['signal']
+		del kwargs['sender']
+	except KeyError:
+		pass
+	wx.CallAfter(save_files_as_new_document, **kwargs)
 #----------------------
-def save_file_as_new_document(parent=None, filename=None, document_type=None, unlock_patient=False, episode=None, **kwargs):
+def save_file_as_new_document(parent=None, filename=None, document_type=None, unlock_patient=False, episode=None, review_as_normal=False):
+	return save_files_as_new_document (
+		parent = parent,
+		filenames = [filename],
+		document_type = document_type,
+		unlock_patient = unlock_patient,
+		episode = episode,
+		review_as_normal = review_as_normal
+	)
+#----------------------
+def save_files_as_new_document(parent=None, filenames=None, document_type=None, unlock_patient=False, episode=None, review_as_normal=False):
 
 	pat = gmPerson.gmCurrentPatient()
 	if not pat.connected:
@@ -133,18 +167,20 @@ def save_file_as_new_document(parent=None, filename=None, document_type=None, un
 		encounter = emr.active_encounter['pk_encounter'],
 		episode = episode['pk_episode']
 	)
-	part = doc.add_part(file = filename)
-	part['filename'] = filename
-	part.save_payload()
+	doc.add_parts_from_files(files = filenames)
+
+	if review_as_normal:
+		doc.set_reviewed(technically_abnormal = False, clinically_relevant = False)
 
 	if unlock_patient:
 		pat.locked = False
 
-	gmDispatcher.send(signal = 'statustext', msg = _('Imported new document from [%s].') % filename, beep = True)
+	gmDispatcher.send(signal = 'statustext', msg = _('Imported new document from %s.') % filenames, beep = True)
 
 	return doc
 #----------------------
 gmDispatcher.connect(signal = u'import_document_from_file', receiver = _save_file_as_new_document)
+gmDispatcher.connect(signal = u'import_document_from_files', receiver = _save_files_as_new_document)
 #============================================================
 class cDocumentCommentPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select a document comment from all existing comments."""
@@ -161,26 +197,38 @@ class cDocumentCommentPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			queries = [u"""
-select *
-from (
-	select distinct on (comment) *
-	from (
-		-- keyed by doc type
-		select comment, comment as pk, 1 as rank
-		from blobs.doc_med
-		where
+SELECT
+	data,
+	field_label,
+	list_label
+FROM (
+	SELECT DISTINCT ON (field_label) *
+	FROM (
+		-- constrained by doc type
+		SELECT
+			comment AS data,
+			comment AS field_label,
+			comment AS list_label,
+			1 AS rank
+		FROM blobs.doc_med
+		WHERE
 			comment %(fragment_condition)s
 			%(ctxt_doc_type)s
 
-		union all
+		UNION ALL
 
-		select comment, comment as pk, 2 as rank
-		from blobs.doc_med
-		where comment %(fragment_condition)s
-	) as q_union
-) as q_distinct
-order by rank, comment
-limit 25"""],
+		SELECT
+			comment AS data,
+			comment AS field_label,
+			comment AS list_label,
+			2 AS rank
+		FROM blobs.doc_med
+		WHERE
+			comment %(fragment_condition)s
+	) AS q_union
+) AS q_distinct
+ORDER BY rank, list_label
+LIMIT 25"""],
 			context = context
 		)
 		mp.setThresholds(3, 5, 7)
@@ -362,16 +410,34 @@ class cDocumentTypeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			queries = [
-u"""select * from ((
-	select pk_doc_type, l10n_type, 1 as rank from blobs.v_doc_type where
-		is_user_defined is True and
+u"""SELECT
+	data,
+	field_label,
+	list_label
+FROM ((
+	SELECT
+		pk_doc_type AS data,
+		l10n_type AS field_label,
+		l10n_type AS list_label,
+		1 AS rank
+	FROM blobs.v_doc_type
+	WHERE
+		is_user_defined IS True
+			AND
 		l10n_type %(fragment_condition)s
-) union (
-	select pk_doc_type, l10n_type, 2 from blobs.v_doc_type where
-		is_user_defined is False and
+) UNION (
+	SELECT
+		pk_doc_type AS data,
+		l10n_type AS field_label,
+		l10n_type AS list_label,
+		2 AS rank
+	FROM blobs.v_doc_type
+	WHERE
+		is_user_defined IS False
+			AND
 		l10n_type %(fragment_condition)s
-)) as q1 order by q1.rank, q1.l10n_type
-"""]
+)) AS q1
+ORDER BY q1.rank, q1.list_label"""]
 			)
 		mp.setThresholds(2, 4, 6)
 
@@ -380,12 +446,25 @@ u"""select * from ((
 
 		self.SetToolTipString(_('Select the document type.'))
 	#--------------------------------------------------------
-	def GetData(self, can_create=False):
-		if self.data is None:
-			if can_create:
-				self.data = gmDocuments.create_document_type(self.GetValue().strip())['pk_doc_type']	# FIXME: error handling
-		return self.data
+	def _create_data(self):
+
+		doc_type = self.GetValue().strip()
+		if doc_type == u'':
+			gmDispatcher.send(signal = u'statustext', msg = _('Cannot create document type without name.'), beep = True)
+			_log.debug('cannot create document type without name')
+			return
+
+		pk = gmDocuments.create_document_type(doc_type)['pk_doc_type']
+		if pk is None:
+			self.data = {}
+		else:
+			self.SetText (
+				value = doc_type,
+				data = pk
+			)
 #============================================================
+from Gnumed.wxGladeWidgets import wxgReviewDocPartDlg
+
 class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 	def __init__(self, *args, **kwds):
 		"""Support parts and docs now.
@@ -400,7 +479,10 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 			self.__reviewing_doc = False
 		elif isinstance(part, gmDocuments.cDocument):
 			self.__doc = part
-			self.__part = self.__doc.parts[0]
+			if len(self.__doc.parts) == 0:
+				self.__part = None
+			else:
+				self.__part = self.__doc.parts[0]
 			self.__reviewing_doc = True
 		else:
 			raise ValueError('<part> must be gmDocuments.cDocument or gmDocuments.cDocumentPart instance, got <%s>' % type(part))
@@ -412,20 +494,20 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 	def __init_ui_data(self):
 		# FIXME: fix this
 		# associated episode (add " " to avoid popping up pick list)
-		self._PhWheel_episode.SetText('%s ' % self.__part['episode'], self.__part['pk_episode'])
-		self._PhWheel_doc_type.SetText(value = self.__part['l10n_type'], data = self.__part['pk_type'])
+		self._PhWheel_episode.SetText('%s ' % self.__doc['episode'], self.__doc['pk_episode'])
+		self._PhWheel_doc_type.SetText(value = self.__doc['l10n_type'], data = self.__doc['pk_type'])
 		self._PhWheel_doc_type.add_callback_on_set_focus(self._on_doc_type_gets_focus)
 		self._PhWheel_doc_type.add_callback_on_lose_focus(self._on_doc_type_loses_focus)
 
 		if self.__reviewing_doc:
-			self._PRW_doc_comment.SetText(gmTools.coalesce(self.__part['doc_comment'], ''))
-			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = self.__part['pk_type'])
+			self._PRW_doc_comment.SetText(gmTools.coalesce(self.__doc['comment'], ''))
+			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = self.__doc['pk_type'])
 		else:
 			self._PRW_doc_comment.SetText(gmTools.coalesce(self.__part['obj_comment'], ''))
 
-		fts = gmDateTime.cFuzzyTimestamp(timestamp = self.__part['date_generated'])
+		fts = gmDateTime.cFuzzyTimestamp(timestamp = self.__doc['clin_when'])
 		self._PhWheel_doc_date.SetText(fts.strftime('%Y-%m-%d'), fts)
-		self._TCTRL_reference.SetValue(gmTools.coalesce(self.__part['ext_ref'], ''))
+		self._TCTRL_reference.SetValue(gmTools.coalesce(self.__doc['ext_ref'], ''))
 		if self.__reviewing_doc:
 			self._TCTRL_filename.Enable(False)
 			self._SPINCTRL_seq_idx.Enable(False)
@@ -448,28 +530,36 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 			self._LCTRL_existing_reviews.SetColumnWidth(col=3, width=wx.LIST_AUTOSIZE_USEHEADER)
 			self._LCTRL_existing_reviews.SetColumnWidth(col=4, width=wx.LIST_AUTOSIZE)
 
-		me = gmPerson.gmCurrentProvider()
-		if self.__part['pk_intended_reviewer'] == me['pk_staff']:
-			msg = _('(you are the primary reviewer)')
+		if self.__part is None:
+			self._ChBOX_review.SetValue(False)
+			self._ChBOX_review.Enable(False)
+			self._ChBOX_abnormal.Enable(False)
+			self._ChBOX_relevant.Enable(False)
+			self._ChBOX_sign_all_pages.Enable(False)
 		else:
-			msg = _('(someone else is the primary reviewer)')
-		self._TCTRL_responsible.SetValue(msg)
+			me = gmPerson.gmCurrentProvider()
+			if self.__part['pk_intended_reviewer'] == me['pk_staff']:
+				msg = _('(you are the primary reviewer)')
+			else:
+				msg = _('(someone else is the primary reviewer)')
+			self._TCTRL_responsible.SetValue(msg)
+			# init my review if any
+			if self.__part['reviewed_by_you']:
+				revs = self.__part.get_reviews()
+				for rev in revs:
+					if rev['is_your_review']:
+						self._ChBOX_abnormal.SetValue(bool(rev[2]))
+						self._ChBOX_relevant.SetValue(bool(rev[3]))
+						break
 
-		# init my review if any
-		if self.__part['reviewed_by_you']:
-			revs = self.__part.get_reviews()
-			for rev in revs:
-				if rev['is_your_review']:
-					self._ChBOX_abnormal.SetValue(bool(rev[2]))
-					self._ChBOX_relevant.SetValue(bool(rev[3]))
-					break
-
-		self._ChBOX_sign_all_pages.SetValue(self.__reviewing_doc)
+			self._ChBOX_sign_all_pages.SetValue(self.__reviewing_doc)
 
 		return True
 	#--------------------------------------------------------
 	def __reload_existing_reviews(self):
 		self._LCTRL_existing_reviews.DeleteAllItems()
+		if self.__part is None:
+			return True
 		revs = self.__part.get_reviews()		# FIXME: this is ugly as sin, it should be dicts, not lists
 		if len(revs) == 0:
 			return True
@@ -516,7 +606,7 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		if pk_episode is None:
 			gmGuiHelpers.gm_show_error (
 				_('Cannot create episode\n [%s]'),
-				_('editing document properties')
+				_('Editing document properties')
 			)
 			return False
 
@@ -539,7 +629,7 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		if not success:
 			gmGuiHelpers.gm_show_error (
 				_('Cannot link the document to episode\n\n [%s]') % epi_name,
-				_('editing document properties')
+				_('Editing document properties')
 			)
 			return False
 
@@ -561,19 +651,34 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 				if self._ChBOX_responsible.GetValue():
 					self.__part['pk_intended_reviewer'] = provider['pk_staff']
 			if msg is not None:
-				gmGuiHelpers.gm_show_error(msg, _('editing document properties'))
+				gmGuiHelpers.gm_show_error(msg, _('Editing document properties'))
 				return False
 
 		# 3) handle "page" specific parts
 		if not self.__reviewing_doc:
 			self.__part['filename'] = gmTools.none_if(self._TCTRL_filename.GetValue().strip(), u'')
-			self.__part['seq_idx'] = gmTools.none_if(self._SPINCTRL_seq_idx.GetValue(), 0)
+			new_idx = gmTools.none_if(self._SPINCTRL_seq_idx.GetValue(), 0)
+			if new_idx in self.__doc['seq_idx_list']:
+				msg = _(
+					'Cannot set page number to [%s] because\n'
+					'another page with this number exists.\n'
+					'\n'
+					'Page numbers in use:\n'
+					'\n'
+					' %s'
+				) % (
+					new_idx,
+					self.__doc['seq_idx_list']
+				)
+				gmGuiHelpers.gm_show_error(msg, _('Editing document part properties'))
+			else:
+				self.__part['seq_idx'] = new_idx
 			self.__part['obj_comment'] = self._PRW_doc_comment.GetValue().strip()
 			success, data = self.__part.save_payload()
 			if not success:
 				gmGuiHelpers.gm_show_error (
 					_('Error saving part properties.'),
-					_('editing document properties')
+					_('Editing document part properties')
 				)
 				return False
 
@@ -599,6 +704,37 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
 		return True
 #============================================================
+def acquire_images_from_capture_device(device=None, calling_window=None):
+
+	_log.debug('acquiring images from [%s]', device)
+
+	# do not import globally since we might want to use
+	# this module without requiring any scanner to be available
+	from Gnumed.pycommon import gmScanBackend
+	try:
+		fnames = gmScanBackend.acquire_pages_into_files (
+			device = device,
+			delay = 5,
+			calling_window = calling_window
+		)
+	except OSError:
+		_log.exception('problem acquiring image from source')
+		gmGuiHelpers.gm_show_error (
+			aMessage = _(
+				'No images could be acquired from the source.\n\n'
+				'This may mean the scanner driver is not properly installed.\n\n'
+				'On Windows you must install the TWAIN Python module\n'
+				'while on Linux and MacOSX it is recommended to install\n'
+				'the XSane package.'
+			),
+			aTitle = _('Acquiring images')
+		)
+		return None
+
+	_log.debug('acquired %s images', len(fnames))
+
+	return fnames
+#------------------------------------------------------------
 from Gnumed.wxGladeWidgets import wxgScanIdxPnl
 
 class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_PluginMixin):
@@ -980,7 +1116,18 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		# - date of generation
 		new_doc['clin_when'] = self._PhWheel_doc_date.GetData().get_pydt()
 		# - external reference
-		ref = gmDocuments.get_ext_ref()
+		cfg = gmCfg.cCfgSQL()
+		generate_uuid = bool (
+			cfg.get2 (
+				option = 'horstspace.scan_index.generate_doc_uuid',
+				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				bias = 'user',
+				default = False
+			)
+		)
+		ref = None
+		if generate_uuid:
+			ref = gmDocuments.get_ext_ref()
 		if ref is not None:
 			new_doc['ext_ref'] = ref
 		# - comment
@@ -1030,7 +1177,6 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		gmHooks.run_hook_script(hook = u'after_new_doc_created')
 
 		# inform user
-		cfg = gmCfg.cCfgSQL()
 		show_id = bool (
 			cfg.get2 (
 				option = 'horstspace.scan_index.show_doc_id',
@@ -1039,8 +1185,11 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			)
 		)
 		wx.EndBusyCursor()
-		if show_id and (ref is not None):
-			msg = _(
+		if show_id:
+			if ref is None:
+				msg = _('Successfully saved the new document.')
+			else:
+				msg = _(
 """The reference ID for the new document is:
 
  <%s>
@@ -1052,7 +1201,7 @@ If you don't care about the ID you can switch
 off this message in the GNUmed configuration.""") % ref
 			gmGuiHelpers.gm_show_info (
 				aMessage = msg,
-				aTitle = _('saving document')
+				aTitle = _('Saving document')
 			)
 		else:
 			gmDispatcher.send(signal='statustext', msg=_('Successfully saved new document.'))
@@ -1075,6 +1224,8 @@ off this message in the GNUmed configuration.""") % ref
 			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
 		return True
 #============================================================
+from Gnumed.wxGladeWidgets import wxgSelectablySortedDocTreePnl
+
 class cSelectablySortedDocTreePnl(wxgSelectablySortedDocTreePnl.wxgSelectablySortedDocTreePnl):
 	"""A panel with a document tree which can be sorted."""
 	#--------------------------------------------------------
@@ -1203,7 +1354,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	def __build_context_menus(self):
 
 		# --- part context menu ---
-		self.__part_context_menu = wx.Menu(title = _('part menu'))
+		self.__part_context_menu = wx.Menu(title = _('Part Actions:'))
 
 		ID = wx.NewId()
 		self.__part_context_menu.Append(ID, _('Display part'))
@@ -1230,7 +1381,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__part_context_menu.AppendSeparator()			# so we can append some items
 
 		# --- doc context menu ---
-		self.__doc_context_menu = wx.Menu(title = _('document menu'))
+		self.__doc_context_menu = wx.Menu(title = _('Document Actions:'))
 
 		ID = wx.NewId()
 		self.__doc_context_menu.Append(ID, _('%s Sign/Edit properties') % u'\u270D')
@@ -1375,7 +1526,12 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 #					rel = ' [%s]' % _('Cave')
 #				else:
 #					rel = ''
-				label = '%s%s (%s)%s' % (
+				f_ext = u''
+				if part['filename'] is not None:
+					f_ext = os.path.splitext(part['filename'])[1].strip('.').strip()
+				if f_ext != u'':
+					f_ext = u' .' + f_ext.upper()
+				label = '%s%s (%s%s)%s' % (
 					gmTools.bool2str (
 						boolean = part['reviewed'] or part['reviewed_by_you'] or part['reviewed_by_intended_reviewer'],
 						true_str = u'',
@@ -1383,6 +1539,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 					),
 					_('part %2s') % part['seq_idx'],
 					gmTools.size2str(part['size']),
+					f_ext,
 					gmTools.coalesce (
 						part['obj_comment'],
 						u'',
@@ -1697,18 +1854,27 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# 0: never
 		# 1: always
 		# 2: if no review by myself exists yet
+		# 3: if no review at all exists yet
+		# 4: if no review by responsible reviewer
 		review_after_display = int(cfg.get2 (
 			option = 'horstspace.document_viewer.review_after_display',
 			workplace = gmSurgery.gmCurrentPractice().active_workplace,
 			bias = 'user',
-			default = 2
+			default = 3
 		))
 		if review_after_display == 1:			# always review
 			self.__review_part(part=part)
 		elif review_after_display == 2:			# review if no review by me exists
 			review_by_me = filter(lambda rev: rev['is_your_review'], part.get_reviews())
 			if len(review_by_me) == 0:
-				self.__review_part(part=part)
+				self.__review_part(part = part)
+		elif review_after_display == 3:
+			if len(part.get_reviews()) == 0:
+				self.__review_part(part = part)
+		elif review_after_display == 4:
+			reviewed_by_responsible = filter(lambda rev: rev['is_review_by_responsible_reviewer'], part.get_reviews())
+			if len(reviewed_by_responsible) == 0:
+				self.__review_part(part = part)
 
 		return True
 	#--------------------------------------------------------
