@@ -315,7 +315,7 @@ limit 25
 			procedure = self._PRW_procedure.GetValue().strip()
 		)
 
-		proc['clin_when'] = self._DPRW_date.data.get_pydt()
+		proc['clin_when'] = self._DPRW_date.GetData().get_pydt()
 		if self._DPRW_end.GetData() is None:
 			proc['clin_end'] = None
 		else:
@@ -323,12 +323,14 @@ limit 25
 		proc['is_ongoing'] = self._CHBOX_ongoing.IsChecked()
 		proc.save()
 
+		proc.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
+
 		self.data = proc
 
 		return True
 	#----------------------------------------------------------------
 	def _save_as_update(self):
-		self.data['clin_when'] = self._DPRW_date.data.get_pydt()
+		self.data['clin_when'] = self._DPRW_date.GetData().get_pydt()
 
 		if self._DPRW_end.GetData() is None:
 			self.data['clin_end'] = None
@@ -350,6 +352,8 @@ limit 25
 		self.data['performed_procedure'] = self._PRW_procedure.GetValue().strip()
 
 		self.data.save()
+		self.data.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
+
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
@@ -361,6 +365,7 @@ limit 25
 		self._PRW_location.SetText()
 		self._PRW_episode.SetText()
 		self._PRW_procedure.SetText()
+		self._PRW_codes.SetText()
 
 		self._PRW_procedure.SetFocus()
 	#----------------------------------------------------------------
@@ -390,6 +395,9 @@ limit 25
 			self._LBL_hospital_details.SetLabel(gmEMRStructItems.cHospitalStay(aPK_obj = self.data['pk_hospital_stay']).format())
 			self._PRW_location.SetText()
 
+		val, data = self._PRW_codes.generic_linked_codes2item_dict(self.data.generic_codes)
+		self._PRW_codes.SetText(val, data)
+
 		self._PRW_procedure.SetFocus()
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
@@ -606,7 +614,7 @@ class cHospitalStayEditAreaPnl(wxgHospitalStayEditAreaPnl.wxgHospitalStayEditAre
 	def _refresh_as_new(self):
 		self._PRW_hospital.SetText(value = u'')
 		self._PRW_episode.SetText(value = u'')
-		self._PRW_admission.SetText(data = pydt.datetime.now())
+		self._PRW_admission.SetText(data = gmDateTime.pydt_now_here())
 		self._PRW_discharge.SetText()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
@@ -789,30 +797,30 @@ class cEncounterTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			queries = [
 u"""
-select pk, l10n_description from (
-	select distinct on (pk) * from (
-		(select
-			pk,
-			_(description) as l10n_description,
-			1 as rank
-		from
+SELECT
+	data,
+	field_label,
+	list_label
+FROM (
+	SELECT DISTINCT ON (data) *
+	FROM (
+		SELECT
+			pk AS data,
+			_(description) AS field_label,
+			case
+				when _(description) = description then _(description)
+				else _(description) || ' (' || description || ')'
+			end AS list_label
+		FROM
 			clin.encounter_type
-		where
+		WHERE
 			_(description) %(fragment_condition)s
-
-		) union all (
-
-		select
-			pk,
-			_(description) as l10n_description,
-			2 as rank
-		from
-			clin.encounter_type
-		where
+				OR
 			description %(fragment_condition)s
-		)
-	) as q_distinct_pk
-) as q_ordered order by rank, l10n_description
+	) AS q_distinct_pk
+) AS q_ordered
+ORDER BY
+	list_label
 """			]
 		)
 		mp.setThresholds(2, 4, 6)
@@ -959,9 +967,17 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		)
 		self._PRW_end.SetText(fts.format_accurately(), data=fts)
 
+		# RFE
 		self._TCTRL_rfe.SetValue(gmTools.coalesce(self.__encounter['reason_for_encounter'], ''))
-		self._TCTRL_aoe.SetValue(gmTools.coalesce(self.__encounter['assessment_of_encounter'], ''))
+		val, data = self._PRW_rfe_codes.generic_linked_codes2item_dict(self.__encounter.generic_codes_rfe)
+		self._PRW_rfe_codes.SetText(val, data)
 
+		# AOE
+		self._TCTRL_aoe.SetValue(gmTools.coalesce(self.__encounter['assessment_of_encounter'], ''))
+		val, data = self._PRW_aoe_codes.generic_linked_codes2item_dict(self.__encounter.generic_codes_aoe)
+		self._PRW_aoe_codes.SetText(val, data)
+
+		# last affirmed
 		if self.__encounter['last_affirmed'] == self.__encounter['started']:
 			self._PRW_end.SetFocus()
 		else:
@@ -979,13 +995,33 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		self._PRW_encounter_type.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
 		self._PRW_encounter_type.Refresh()
 
-		if (not self._PRW_start.is_valid_timestamp()) or (self._PRW_start.GetValue().strip() == u''):
+		# start
+		if self._PRW_start.GetValue().strip() == u'':
+			self._PRW_start.SetBackgroundColour('pink')
+			self._PRW_start.Refresh()
 			self._PRW_start.SetFocus()
 			return False
+		if not self._PRW_start.is_valid_timestamp():
+			self._PRW_start.SetBackgroundColour('pink')
+			self._PRW_start.Refresh()
+			self._PRW_start.SetFocus()
+			return False
+		self._PRW_start.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		self._PRW_start.Refresh()
 
-		if (not self._PRW_end.is_valid_timestamp()) or (self._PRW_end.GetValue().strip() == u''):
+		# last_affirmed
+		if self._PRW_end.GetValue().strip() == u'':
+			self._PRW_end.SetBackgroundColour('pink')
+			self._PRW_end.Refresh()
 			self._PRW_end.SetFocus()
 			return False
+		if not self._PRW_end.is_valid_timestamp():
+			self._PRW_end.SetBackgroundColour('pink')
+			self._PRW_end.Refresh()
+			self._PRW_end.SetFocus()
+			return False
+		self._PRW_end.SetBackgroundColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
+		self._PRW_end.Refresh()
 
 		return True
 	#--------------------------------------------------------
@@ -1000,6 +1036,9 @@ class cEncounterEditAreaPnl(wxgEncounterEditAreaPnl.wxgEncounterEditAreaPnl):
 		self.__encounter['assessment_of_encounter'] = gmTools.none_if(self._TCTRL_aoe.GetValue().strip(), u'')
 		self.__encounter.save_payload()			# FIXME: error checking
 
+		self.__encounter.generic_codes_rfe = [ c['data'] for c in self._PRW_rfe_codes.GetData() ]
+		self.__encounter.generic_codes_aoe = [ c['data'] for c in self._PRW_aoe_codes.GetData() ]
+
 		return True
 #----------------------------------------------------------------
 # FIXME: use generic dialog 2
@@ -1269,12 +1308,27 @@ class cEpisodeDescriptionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	def __init__(self, *args, **kwargs):
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
-			queries = [u"""
-				select distinct on (description) description, description, 1
-				from clin.episode
-				where description %(fragment_condition)s
-				order by description
-				limit 30"""
+			queries = [
+u"""
+SELECT DISTINCT ON (description)
+	description
+		AS data,
+	description
+		AS field_label,
+	description || ' ('
+	|| CASE
+		WHEN is_open IS TRUE THEN _('ongoing')
+		ELSE _('closed')
+	   END
+	|| ')'
+		AS list_label
+FROM
+	clin.episode
+WHERE
+	description %(fragment_condition)s
+ORDER BY description
+LIMIT 30
+"""
 			]
 		)
 		gmPhraseWheel.cPhraseWheel.__init__(self, *args, **kwargs)
@@ -1304,11 +1358,14 @@ class cEpisodeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 u"""(
 
 select
-	pk_episode,
+	pk_episode
+		as data,
+	description
+		as field_label,
 	coalesce (
 		description || ' - ' || health_issue,
 		description
-	) as description,
+	) as list_label,
 	1 as rank
 from
   	clin.v_pat_episodes
@@ -1320,11 +1377,14 @@ where
 ) union all (
 
 select
-	pk_episode,
+	pk_episode
+		as data,
+	description
+		as field_label,
 	coalesce (
 		description || _(' (closed)') || ' - ' || health_issue,
 		description || _(' (closed)')
-	) as description,
+	) as list_label,
 	2 as rank
 from
 	clin.v_pat_episodes
@@ -1334,7 +1394,8 @@ where
 	%(ctxt_pat)s
 
 )
-order by rank, description
+
+order by rank, list_label
 limit 30"""
 ],
 			context = ctxt
@@ -1376,8 +1437,7 @@ limit 30"""
 	#--------------------------------------------------------
 	def GetData(self, can_create=False, as_instance=False, is_open=False):
 		self.__is_open_for_create_data = is_open		# used (only) in _create_data()
-		gmPhraseWheel.cPhraseWheel.GetData(self, can_create = can_create, as_instance = as_instance)
-		return self.data
+		return gmPhraseWheel.cPhraseWheel.GetData(self, can_create = can_create, as_instance = as_instance)
 	#--------------------------------------------------------
 	def _create_data(self):
 
@@ -1395,12 +1455,15 @@ limit 30"""
 		emr = pat.get_emr()
 		epi = emr.add_episode(episode_name = epi_name, is_open = self.__is_open_for_create_data)
 		if epi is None:
-			self.data = None
+			self.data = {}
 		else:
-			self.data = epi['pk_episode']
+			self.SetText (
+				value = epi_name,
+				data = epi['pk_episode']
+			)
 	#--------------------------------------------------------
 	def _data2instance(self):
-		return gmEMRStructItems.cEpisode(aPK_obj = self.data)
+		return gmEMRStructItems.cEpisode(aPK_obj = self.GetData())
 	#--------------------------------------------------------
 	# internal API
 	#--------------------------------------------------------
@@ -1456,9 +1519,9 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 		emr = pat.get_emr()
 
 		epi = emr.add_episode(episode_name = self._PRW_description.GetValue().strip())
-		epi['summary'] = self._TCTRL_summary.GetValue().strip()
+		epi['summary'] = self._TCTRL_status.GetValue().strip()
 		epi['episode_open'] = not self._CHBOX_closed.IsChecked()
-		epi['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
+		epi['diagnostic_certainty_classification'] = self._PRW_certainty.GetData()
 
 		issue_name = self._PRW_issue.GetValue().strip()
 		if len(issue_name) != 0:
@@ -1478,15 +1541,17 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 
 		epi.save()
 
+		epi.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
+
 		self.data = epi
 		return True
 	#----------------------------------------------------------------
 	def _save_as_update(self):
 
 		self.data['description'] = self._PRW_description.GetValue().strip()
-		self.data['summary'] = self._TCTRL_summary.GetValue().strip()
+		self.data['summary'] = self._TCTRL_status.GetValue().strip()
 		self.data['episode_open'] = not self._CHBOX_closed.IsChecked()
-		self.data['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
+		self.data['diagnostic_certainty_classification'] = self._PRW_certainty.GetData()
 
 		issue_name = self._PRW_issue.GetValue().strip()
 		if len(issue_name) == 0:
@@ -1506,6 +1571,8 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 				return False
 
 		self.data.save()
+		self.data.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
+
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
@@ -1516,9 +1583,10 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 		self._TCTRL_patient.SetValue(ident.get_description_gender())
 		self._PRW_issue.SetText()
 		self._PRW_description.SetText()
-		self._TCTRL_summary.SetValue(u'')
-		self._PRW_classification.SetText()
+		self._TCTRL_status.SetValue(u'')
+		self._PRW_certainty.SetText()
 		self._CHBOX_closed.SetValue(False)
+		self._PRW_codes.SetText()
 	#----------------------------------------------------------------
 	def _refresh_from_existing(self):
 		ident = gmPerson.cIdentity(aPK_obj = self.data['pk_patient'])
@@ -1529,12 +1597,15 @@ class cEpisodeEditAreaPnl(gmEditArea.cGenericEditAreaMixin, wxgEpisodeEditAreaPn
 
 		self._PRW_description.SetText(self.data['description'], data=self.data['description'])
 
-		self._TCTRL_summary.SetValue(gmTools.coalesce(self.data['summary'], u''))
+		self._TCTRL_status.SetValue(gmTools.coalesce(self.data['summary'], u''))
 
 		if self.data['diagnostic_certainty_classification'] is not None:
-			self._PRW_classification.SetData(data = self.data['diagnostic_certainty_classification'])
+			self._PRW_certainty.SetData(data = self.data['diagnostic_certainty_classification'])
 
 		self._CHBOX_closed.SetValue(not self.data['episode_open'])
+
+		val, data = self._PRW_codes.generic_linked_codes2item_dict(self.data.generic_codes)
+		self._PRW_codes.SetText(val, data)
 	#----------------------------------------------------------------
 	def _refresh_as_new_from_existing(self):
 		self._refresh_as_new()
@@ -1604,23 +1675,41 @@ class cIssueSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 
 		mp = gmMatchProvider.cMatchProvider_SQL2 (
 			# FIXME: consider clin.health_issue.clinically_relevant
-			queries = [u"""
-(select pk_health_issue, description, 1
-	from clin.v_health_issues where
-		is_active is true and
-		description %(fragment_condition)s and
+			queries = [
+u"""
+SELECT
+	data,
+	field_label,
+	list_label
+FROM ((
+	SELECT
+		pk_health_issue AS data,
+		description AS field_label,
+		description AS list_label
+	FROM clin.v_health_issues
+	WHERE
+		is_active IS true
+			AND
+		description %(fragment_condition)s
+			AND
 		%(ctxt_pat)s
-	order by description)
 
-union
+	) UNION (
 
-(select pk_health_issue, description || _(' (inactive)'), 2
-	from clin.v_health_issues where
-		is_active is false and
-		description %(fragment_condition)s and
+	SELECT
+		pk_health_issue AS data,
+		description AS field_label,
+		description || _(' (inactive)') AS list_label
+	FROM clin.v_health_issues
+	WHERE
+		is_active IS false
+			AND
+		description %(fragment_condition)s
+			AND
 		%(ctxt_pat)s
-	order by description)"""
-			],
+)) AS union_query
+ORDER BY
+	list_label"""],
 			context = ctxt
 		)
 
@@ -1782,6 +1871,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		self._PRW_year_noted.Enable(True)
 
+		self._PRW_codes.add_callback_on_lose_focus(self._on_leave_codes)
+
 		self.data = issue
 	#----------------------------------------------------------------
 	# generic Edit Area mixin API
@@ -1819,8 +1910,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			side += u'd'
 		issue['laterality'] = side
 
-		issue['summary'] = self._TCTRL_summary.GetValue().strip()
-		issue['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
+		issue['summary'] = self._TCTRL_status.GetValue().strip()
+		issue['diagnostic_certainty_classification'] = self._PRW_certainty.GetData()
 		issue['grouping'] = self._PRW_grouping.GetValue().strip()
 		issue['is_active'] = self._ChBOX_active.GetValue()
 		issue['clinically_relevant'] = self._ChBOX_relevant.GetValue()
@@ -1833,6 +1924,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		issue.save()
 
+		issue.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
+
 		self.data = issue
 		return True
 	#----------------------------------------------------------------
@@ -1847,8 +1940,8 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			side += u'd'
 		self.data['laterality'] = side
 
-		self.data['summary'] = self._TCTRL_summary.GetValue().strip()
-		self.data['diagnostic_certainty_classification'] = self._PRW_classification.GetData()
+		self.data['summary'] = self._TCTRL_status.GetValue().strip()
+		self.data['diagnostic_certainty_classification'] = self._PRW_certainty.GetData()
 		self.data['grouping'] = self._PRW_grouping.GetValue().strip()
 		self.data['is_active'] = bool(self._ChBOX_active.GetValue())
 		self.data['clinically_relevant'] = bool(self._ChBOX_relevant.GetValue())
@@ -1860,22 +1953,23 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 			self.data['age_noted'] = age_noted
 
 		self.data.save()
+		self.data.generic_codes = [ c['data'] for c in self._PRW_codes.GetData() ]
 
-		# FIXME: handle is_operation
 		return True
 	#----------------------------------------------------------------
 	def _refresh_as_new(self):
 		self._PRW_condition.SetText()
 		self._ChBOX_left.SetValue(0)
 		self._ChBOX_right.SetValue(0)
-		self._PRW_classification.SetText()
+		self._PRW_codes.SetText()
+		self._on_leave_codes()
+		self._PRW_certainty.SetText()
 		self._PRW_grouping.SetText()
-		self._TCTRL_summary.SetValue(u'')
+		self._TCTRL_status.SetValue(u'')
 		self._PRW_age_noted.SetText()
 		self._PRW_year_noted.SetText()
 		self._ChBOX_active.SetValue(0)
 		self._ChBOX_relevant.SetValue(1)
-		self._ChBOX_is_operation.SetValue(0)
 		self._ChBOX_confidential.SetValue(0)
 		self._ChBOX_caused_death.SetValue(0)
 
@@ -1894,9 +1988,14 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 		else:
 			self._ChBOX_right.SetValue(1)
 
-		self._PRW_classification.SetData(data = self.data['diagnostic_certainty_classification'])
+		val, data = self._PRW_codes.generic_linked_codes2item_dict(self.data.generic_codes)
+		self._PRW_codes.SetText(val, data)
+		self._on_leave_codes()
+
+		if self.data['diagnostic_certainty_classification'] is not None:
+			self._PRW_certainty.SetData(data = self.data['diagnostic_certainty_classification'])
 		self._PRW_grouping.SetText(gmTools.coalesce(self.data['grouping'], u''))
-		self._TCTRL_summary.SetValue(gmTools.coalesce(self.data['summary'], u''))
+		self._TCTRL_status.SetValue(gmTools.coalesce(self.data['summary'], u''))
 
 		if self.data['age_noted'] is None:
 			self._PRW_age_noted.SetText()
@@ -1908,7 +2007,6 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 
 		self._ChBOX_active.SetValue(self.data['is_active'])
 		self._ChBOX_relevant.SetValue(self.data['clinically_relevant'])
-		self._ChBOX_is_operation.SetValue(0)		# FIXME
 		self._ChBOX_confidential.SetValue(self.data['is_confidential'])
 		self._ChBOX_caused_death.SetValue(self.data['is_cause_of_death'])
 
@@ -1923,6 +2021,12 @@ limit 50""" % gmPerson.gmCurrentPatient().ID
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
+	def _on_leave_codes(self, *args, **kwargs):
+		if not self._PRW_codes.IsModified():
+			return True
+
+		self._TCTRL_code_details.SetValue(u'- ' + u'\n- '.join([ c['list_label'] for c in self._PRW_codes.GetData() ]))
+	#--------------------------------------------------------
 	def _on_leave_age_noted(self, *args, **kwargs):
 
 		if not self._PRW_age_noted.IsModified():
@@ -2055,7 +2159,7 @@ if __name__ == '__main__':
 	class testapp (wx.App):
 			"""
 			Test application for testing EMR struct widgets
-			"""			
+			"""
 			#--------------------------------------------------------
 			def OnInit (self):
 				"""
