@@ -6,14 +6,17 @@ __version__ = "$Revision: 1.135 $"
 __author__ = "R.Terry, K.Hilbert"
 
 #======================================================================
-import logging, datetime as pydt
+import sys
+import logging
+import datetime as pydt
 
 
 import wx
 
 
-from Gnumed.pycommon import gmDispatcher, gmExceptions
-from Gnumed.wxGladeWidgets import wxgGenericEditAreaDlg, wxgGenericEditAreaDlg2
+if __name__ == '__main__':
+	sys.path.insert(0, '../../')
+from Gnumed.pycommon import gmDispatcher
 
 
 _log = logging.getLogger('gm.ui')
@@ -24,6 +27,8 @@ edit_area_modes = ['new', 'edit', 'new_from_existing']
 class cGenericEditAreaMixin(object):
 	"""Mixin for edit area panels providing generic functionality.
 
+	**************** start of template ****************
+
 #====================================================================
 # Class definition:
 
@@ -57,6 +62,10 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 	# generic Edit Area mixin API
 	#----------------------------------------------------------------
 	def _valid_for_save(self):
+
+		# its best to validate bottom -> top such that the
+		# cursor ends up in the topmost failing field
+
 		# remove when implemented:
 		return False
 
@@ -65,12 +74,14 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if self._TCTRL_xxx.GetValue().strip() == u'':
 			validity = False
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = False)
+			self._TCTRL_xxx.SetFocus()
 		else:
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = True)
 
 		if self._PRW_xxx.GetData() is None:
 			validity = False
 			self._PRW_xxx.display_as_valid(False)
+			self._PRW_xxx.SetFocus()
 		else:
 			self._PRW_xxx.display_as_valid(True)
 
@@ -109,16 +120,18 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 	def _refresh_from_existing(self):
 		pass
 	#----------------------------------------------------------------
+
+	**************** end of template ****************
 	"""
 	def __init__(self):
 		self.__mode = 'new'
 		self.__data = None
 		self.successful_save_msg = None
-		self._refresh_as_new()
 		self.__tctrl_validity_colors = {
 			True: wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW),
 			False: 'pink'
 		}
+		self._refresh_as_new()
 	#----------------------------------------------------------------
 	def _get_mode(self):
 		return self.__mode
@@ -163,6 +176,9 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if not self._valid_for_save():
 			return False
 
+		# remove messages about previous invalid save attempts
+		gmDispatcher.send(signal = 'statustext', msg = u'')
+
 		if self.__mode in ['new', 'new_from_existing']:
 			if self._save_as_new():
 				self.mode = 'edit'
@@ -208,6 +224,8 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		ctrl.SetBackgroundColour(self.__tctrl_validity_colors[valid])
 		ctrl.Refresh()
 #====================================================================
+from Gnumed.wxGladeWidgets import wxgGenericEditAreaDlg2
+
 class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 	"""Dialog for parenting edit area panels with save/clear/next/cancel"""
 
@@ -351,6 +369,8 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 	left_extra_button = property(lambda x:x, _set_left_extra_button)
 #====================================================================
 # DEPRECATED:
+from Gnumed.wxGladeWidgets import wxgGenericEditAreaDlg
+
 class cGenericEditAreaDlg(wxgGenericEditAreaDlg.wxgGenericEditAreaDlg):
 	"""Dialog for parenting edit area with save/clear/cancel"""
 
@@ -431,7 +451,7 @@ class cEditAreaPopup(wx.Dialog):
 		edit_area = None
 	):
 		if not isinstance(edit_area, cEditArea2):
-			raise gmExceptions.ConstructorError, '<edit_area> must be of type cEditArea2 but is <%s>' % type(edit_area)
+			raise TypeError('<edit_area> must be of type cEditArea2 but is <%s>' % type(edit_area))
 		wx.Dialog.__init__(self, parent, id, title, pos, size, style, name)
 		self.__wxID_BTN_SAVE = wx.NewId()
 		self.__wxID_BTN_RESET = wx.NewId()
