@@ -1,3 +1,4 @@
+#  coding: utf8
 """GNUmed macro primitives.
 
 This module implements functions a macro can legally use.
@@ -25,8 +26,13 @@ from Gnumed.pycommon import gmExceptions
 from Gnumed.pycommon import gmCfg2
 from Gnumed.pycommon import gmDateTime
 
-from Gnumed.business import gmPerson, gmDemographicRecord, gmMedication, gmPathLab, gmPersonSearch
-from Gnumed.business import gmVaccination, gmPersonSearch
+from Gnumed.business import gmPerson
+from Gnumed.business import gmDemographicRecord
+from Gnumed.business import gmMedication
+from Gnumed.business import gmPathLab
+from Gnumed.business import gmPersonSearch
+from Gnumed.business import gmVaccination
+from Gnumed.business import gmPersonSearch
 
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmNarrativeWidgets
@@ -52,17 +58,18 @@ known_placeholders = [
 	'soap_p',
 	u'client_version',
 	u'current_provider',
+	u'primary_praxis_provider',			# primary provider for current patient in this praxis
 	u'allergy_state'
 ]
 
 
-# those must satisfy the pattern "$name::args::optional length$" when used
+# those must satisfy the pattern "$<name::args::(optional) max string length>$" when used
 known_variant_placeholders = [
 	u'soap',
-	u'progress_notes',			# "data" holds: categories//template
-								# 	categories: string with "soap ", " " == None == admin
+	u'progress_notes',			# "args" holds: categories//template
+								# 	categories: string with 'soap '; ' ' == None == admin
 								#	template:	u'something %s something'		(do not include // in template !)
-	u'emr_journal',				# "data" format:   <categories>//<template>//<line length>//<time range>//<target format>
+	u'emr_journal',				# "args" format:   <categories>//<template>//<line length>//<time range>//<target format>
 								#	categories:	   string with any of "s", "o", "a", "p", " ";
 								#				   (" " == None == admin category)
 								#	template:	   something %s something else
@@ -71,24 +78,37 @@ known_variant_placeholders = [
 								#	time range:	   the number of weeks going back in time
 								#	target format: "tex" or anything else, if "tex", data will be tex-escaped
 	u'date_of_birth',
-	u'adr_street',				# "data" holds: type of address
+
+	u'patient_address',			# "args": <type of address>//<optional formatting template>
+	u'adr_street',				# "args" holds: type of address
 	u'adr_number',
 	u'adr_location',
 	u'adr_postcode',
-	u'gender_mapper',			# "data" holds: value for male // value for female
-	u'current_meds',			# "data" holds: line template
-	u'current_meds_table',		# "data" holds: format, options
-	u'current_meds_notes',		# "data" holds: format, options
-	u'lab_table',				# "data" holds: format (currently "latex" only)
-	u'latest_vaccs_table',		# "data" holds: format, options
-	u'today',					# "data" holds: strftime format
-	u'tex_escape',				# "data" holds: string to escape
-	u'allergies',				# "data" holds: line template, one allergy per line
-	u'allergy_list',			# "data" holds: template per allergy, allergies on one line
-	u'problems',				# "data" holds: line template, one problem per line
-	u'name',					# "data" holds: template for name parts arrangement
-	u'free_text',				# show a dialog for entering some free text
-	u'soap_for_encounters'		# "data" holds: soap cats // strftime date format
+	u'adr_region',
+	u'adr_country',
+
+	u'patient_comm',						# args: comm channel type as per database
+	u'external_id',							# args: <type of ID>//<issuer of ID>
+	u'gender_mapper',						# "args" holds: <value when person is male> // <is female> // <is other>
+											#				eg. "male//female//other"
+											#				or: "Lieber Patient//Liebe Patientin"
+	u'current_meds',						# "args" holds: line template
+	u'current_meds_table',					# "args" holds: format, options
+											#				currently only "latex"
+	u'current_meds_notes',					# "args" holds: format, options
+	u'lab_table',							# "args" holds: format (currently "latex" only)
+	u'latest_vaccs_table',					# "args" holds: format, options
+	u'today',								# "args" holds: strftime format
+	u'tex_escape',							# "args" holds: string to escape
+	u'allergies',							# "args" holds: line template, one allergy per line
+	u'allergy_list',						# "args" holds: template per allergy, allergies on one line
+	u'problems',							# "args" holds: line template, one problem per line
+	u'name',								# "args" holds: template for name parts arrangement
+	u'free_text',							# show a dialog for entering some free text
+	u'soap_for_encounters',					# "args" holds: soap cats // strftime date format
+	u'encounter_list',						# "args" holds: per-encounter template, each ends up on one line
+	u'current_provider_external_id',		# args: <type of ID>//<issuer of ID>
+	u'primary_praxis_provider_external_id'	# args: <type of ID>//<issuer of ID>
 ]
 
 default_placeholder_regex = r'\$<.+?>\$'				# this one works (except that OOo cannot be non-greedy |-( )
@@ -110,18 +130,14 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 	- is passed to the forms handling code, for example
 
 	Note that this cannot be called from a non-gui thread unless
-	wrapped in wx.CallAfter.
+	wrapped in wx.CallAfter().
 
-	There are currently three types of placeholders:
+	There are currently two types of placeholders:
 
 	simple static placeholders
 		- those are listed in known_placeholders
 		- they are used as-is
 
-	extended static placeholders
-		- those are like the static ones but have "::::<NUMBER>" appended
-		  where <NUMBER> is the maximum length
-
 	variant placeholders
 		- those are listed in known_variant_placeholders
 		- they are parsed into placeholder, data, and maximum length
@@ -258,6 +274,24 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 			u'%s' % self.__class__.__name__
 		)
 	#--------------------------------------------------------
+	def _get_primary_praxis_provider(self):
+		prov = self.pat.primary_provider
+		if prov is None:
+			return self._get_current_provider()
+
+		title = gmTools.coalesce (
+			prov['title'],
+			gmPerson.map_gender2salutation(prov['gender'])
+		)
+
+		tmp = u'%s %s. %s' % (
+			title,
+			prov['firstnames'][:1],
+			prov['lastnames']
+		)
+
+		return tmp
+	#--------------------------------------------------------
 	def _get_current_provider(self):
 		prov = gmPerson.gmCurrentProvider()
 
@@ -311,9 +345,30 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 	client_version = property(_get_client_version, _setter_noop)
 
 	current_provider = property(_get_current_provider, _setter_noop)
+	primary_praxis_provider = property(_get_primary_praxis_provider, _setter_noop)
 	#--------------------------------------------------------
 	# variant handlers
 	#--------------------------------------------------------
+	def _get_variant_encounter_list(self, data=None):
+
+		encounters = gmEMRStructWidgets.select_encounters(single_selection = False)
+		if not encounters:
+			return u''
+
+		template = data
+
+		lines = []
+		for enc in encounters:
+			try:
+				lines.append(template % enc)
+			except:
+				lines.append(u'error formatting encounter')
+				_log.exception('problem formatting encounter list')
+				_log.error('template: %s', template)
+				_log.error('encounter: %s', encounter)
+
+		return u'\n'.join(lines)
+	#--------------------------------------------------------
 	def _get_variant_soap_for_encounters(self, data=None):
 		"""Select encounters from list and format SOAP thereof.
 
@@ -514,9 +569,51 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 
 		return other_value
 	#--------------------------------------------------------
+	# address related placeholders
+	#--------------------------------------------------------
+	def _get_variant_patient_address(self, data=u''):
+
+		data_parts = data.split(u'//')
+
+		if data_parts[0].strip() == u'':
+			adr_type = u'home'
+		else:
+			adr_type = data_parts[0]
+
+		template = _('%(street)s %(number)s, %(postcode)s %(urb)s, %(l10n_state)s, %(l10n_country)s')
+		if len(data_parts) > 1:
+			if data_parts[1].strip() != u'':
+				template = data_parts[1]
+
+		adrs = self.pat.get_addresses(address_type = adr_type)
+		if len(adrs) == 0:
+			return _('no address for type [%s]') % adr_type
+
+		adr = adrs[0]
+		data = {
+			'street': adr['street'],
+			'notes_street': gmTools.coalesce(adr['notes_street'], u''),
+			'postcode': adr['postcode'],
+			'number': adr['number'],
+			'subunit': gmTools.coalesce(adr['subunit'], u''),
+			'notes_subunit': gmTools.coalesce(adr['notes_subunit'], u''),
+			'urb': adr['urb'],
+			'suburb': gmTools.coalesce(adr['suburb'], u''),
+			'l10n_state': adr['l10n_state'],
+			'l10n_country': adr['l10n_country'],
+			'code_state': adr['code_state'],
+			'code_country': adr['code_country']
+		}
+
+		try:
+			return template % data
+		except StandardError:
+			_log.exception('error formatting address')
+			_log.error('template: %s', template)
+
+		return None
+	#--------------------------------------------------------
 	def _get_variant_adr_street(self, data=u'?'):
-#		if data == u'?':
-#			types = xxxxxxxxxxx
 		adrs = self.pat.get_addresses(address_type=data)
 		if len(adrs) == 0:
 			return _('no street for address type [%s]') % data
@@ -535,11 +632,94 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 		return adrs[0]['urb']
 	#--------------------------------------------------------
 	def _get_variant_adr_postcode(self, data=u'?'):
-		adrs = self.pat.get_addresses(address_type=data)
+		adrs = self.pat.get_addresses(address_type = data)
 		if len(adrs) == 0:
 			return _('no postcode for address type [%s]') % data
 		return adrs[0]['postcode']
 	#--------------------------------------------------------
+	def _get_variant_adr_region(self, data=u'?'):
+		adrs = self.pat.get_addresses(address_type = data)
+		if len(adrs) == 0:
+			return _('no region for address type [%s]') % data
+		return adrs[0]['l10n_state']
+	#--------------------------------------------------------
+	def _get_variant_adr_country(self, data=u'?'):
+		adrs = self.pat.get_addresses(address_type = data)
+		if len(adrs) == 0:
+			return _('no country for address type [%s]') % data
+		return adrs[0]['l10n_country']
+	#--------------------------------------------------------
+	def _get_variant_patient_comm(self, data=u'?'):
+		comms = self.pat.get_comm_channels(comm_medium = data)
+		if len(comms) == 0:
+			return _('no URL for comm channel [%s]') % data
+		return comms[0]['url']
+	#--------------------------------------------------------
+	def _get_variant_current_provider_external_id(self, data=u''):
+		data_parts = data.split(u'//')
+		if len(data_parts) < 2:
+			return None
+
+		id_type = data_parts[0].strip()
+		if id_type == u'':
+			return None
+
+		issuer = data_parts[1].strip()
+		if issuer == u'':
+			return None
+
+		prov = gmPerson.gmCurrentProvider()
+		ids = prov.identity.get_external_ids(id_type = id_type, issuer = issuer)
+
+		if len(ids) == 0:
+			return _('no external ID [%s] by [%s]') % (id_type, issuer)
+
+		return ids[0]['value']
+	#--------------------------------------------------------
+	def _get_variant_primary_praxis_provider_external_id(self, data=u''):
+		data_parts = data.split(u'//')
+		if len(data_parts) < 2:
+			return None
+
+		id_type = data_parts[0].strip()
+		if id_type == u'':
+			return None
+
+		issuer = data_parts[1].strip()
+		if issuer == u'':
+			return None
+
+		prov = self.pat.primary_provider
+		if prov is None:
+			return _('no primary in-praxis provider')
+
+		ids = prov.identity.get_external_ids(id_type = id_type, issuer = issuer)
+
+		if len(ids) == 0:
+			return _('no external ID [%s] by [%s]') % (id_type, issuer)
+
+		return ids[0]['value']
+	#--------------------------------------------------------
+	def _get_variant_external_id(self, data=u''):
+		data_parts = data.split(u'//')
+		if len(data_parts) < 2:
+			return None
+
+		id_type = data_parts[0].strip()
+		if id_type == u'':
+			return None
+
+		issuer = data_parts[1].strip()
+		if issuer == u'':
+			return None
+
+		ids = self.pat.get_external_ids(id_type = id_type, issuer = issuer)
+
+		if len(ids) == 0:
+			return _('no external ID [%s] by [%s]') % (id_type, issuer)
+
+		return ids[0]['value']
+	#--------------------------------------------------------
 	def _get_variant_allergy_list(self, data=None):
 		if data is None:
 			return [_('template is missing')]
@@ -648,7 +828,12 @@ class gmPlaceholderHandler(gmBorg.cBorg):
 		#	format:	tex (only, currently)
 		#	message: shown in input dialog, must not contain "//" or "::"
 
-		format, msg = data.split('//')
+		data_parts = data.split('//')
+		format = data_parts[0]
+		if len(data_parts) > 1:
+			msg = data_parts[1]
+		else:
+			msg = _('generic text')
 
 		dlg = gmGuiHelpers.cMultilineTextEntryDlg (
 			None,
@@ -824,8 +1009,7 @@ class cMacroPrimitives:
 		searcher = gmPersonSearch.cPatientSearcher_SQL()
 		if type(search_params) == types.DictType:
 			idents = searcher.get_identities(search_dict=search_params)
-			print "must use dto, not search_dict"
-			print xxxxxxxxxxxxxxxxx
+			raise StandardError("must use dto, not search_dict")
 		else:
 			idents = searcher.get_identities(search_term=search_params)
 		if idents is None:
@@ -1100,14 +1284,28 @@ if __name__ == '__main__':
 	#--------------------------------------------------------
 	def test_placeholder():
 
-		#ph = u'emr_journal::soap //%(date)s  %(modified_by)s  %(soap_cat)s  %(narrative)s//30::'
-		#ph = u'free_text::latex//placeholder test::9999'
-		#ph = u'soap_for_encounters:://::9999'
-		ph = u'soap_a'
+		phs = [
+			#u'emr_journal::soap //%(date)s  %(modified_by)s  %(soap_cat)s  %(narrative)s//30::',
+			#u'free_text::tex//placeholder test::9999',
+			#u'soap_for_encounters:://::9999',
+			#u'soap_a',,
+			#u'encounter_list::%(started)s: %(assessment_of_encounter)s::30',
+			#u'patient_comm::homephone::1234',
+			#u'patient_address::home//::1234',
+			#u'adr_region::home::1234',
+			#u'adr_country::home::1234',
+			#u'external_id::Starfleet Serial Number//Star Fleet Central Staff Office::1234',
+			#u'primary_praxis_provider',
+			#u'current_provider',
+			#u'current_provider_external_id::Starfleet Serial Number//Star Fleet Central Staff Office::1234',
+			#u'current_provider_external_id::LANR//LÄK::1234'
+			u'primary_praxis_provider_external_id::LANR//LÄK::1234'
+		]
 
 		handler = gmPlaceholderHandler()
 		handler.debug = True
 
+		gmPerson.set_current_provider_to_logged_on_user()
 		pat = gmPersonSearch.ask_for_patient()
 		if pat is None:
 			return
@@ -1115,7 +1313,8 @@ if __name__ == '__main__':
 		gmPatSearchWidgets.set_active_patient(patient = pat)
 
 		app = wx.PyWidgetTester(size = (200, 50))
-		print u'%s => %s' % (ph, handler[ph])
+		for ph in phs:
+			print u'%s => %s' % (ph, handler[ph])
 	#--------------------------------------------------------
 
 	#test_placeholders()
