@@ -4,7 +4,7 @@
 __version__ = "$Revision: 1.33 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import logging, sys, os.path, webbrowser, decimal
+import logging, sys, os.path, decimal
 
 
 import wx, wx.grid
@@ -13,7 +13,7 @@ import wx, wx.grid
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
 from Gnumed.pycommon import gmDispatcher, gmCfg, gmShellAPI, gmTools, gmDateTime
-from Gnumed.pycommon import gmMatchProvider, gmI18N, gmPrinting, gmCfg2
+from Gnumed.pycommon import gmMatchProvider, gmI18N, gmPrinting, gmCfg2, gmNetworkTools
 from Gnumed.business import gmPerson, gmATC, gmSurgery, gmMedication, gmForms
 from Gnumed.wxpython import gmGuiHelpers, gmRegetMixin, gmAuthWidgets, gmEditArea, gmMacro
 from Gnumed.wxpython import gmCfgWidgets, gmListWidgets, gmPhraseWheel, gmFormWidgets
@@ -80,6 +80,7 @@ def get_drug_database(parent = None):
 		)
 		if default_db is None:
 			return None
+		drug_db = gmMedication.drug_data_source_interfaces[default_db]()
 
 	pat = gmPerson.gmCurrentPatient()
 	if pat.connected:
@@ -716,7 +717,7 @@ class cSubstancePhraseWheel(gmPhraseWheel.cPhraseWheel):
 def manage_components_of_branded_drug(parent=None, brand=None):
 
 	if brand is not None:
-		if brand['is_in_use']:
+		if brand.is_in_use_by_patients:
 			gmGuiHelpers.gm_show_info (
 				aTitle = _('Managing components of a drug'),
 				aMessage = _(
@@ -789,7 +790,7 @@ def manage_branded_drugs(parent=None, ignore_OK_button=False):
 		tt += u'\n'
 		tt += u'%s%s%s\n' % (
 			gmTools.bool2subst(brand.is_vaccine, u'%s, ' % _('Vaccine'), u''),
-			u'%s, ' % gmTools.bool2subst(brand['is_in_use'], _('in use'), _('not in use')),
+			u'%s, ' % gmTools.bool2subst(brand.is_in_use_by_patients, _('in use'), _('not in use')),
 			gmTools.bool2subst(brand['is_fake_brand'], _('fake'), u'')
 		)
 		tt += gmTools.coalesce(brand['atc'], u'', _('ATC: %s\n'))
@@ -798,20 +799,21 @@ def manage_branded_drugs(parent=None, ignore_OK_button=False):
 			tt += u'- %s' % u'\n- '.join(brand['components'])
 		return tt
 	#------------------------------------------------------------
-	def edit(brand=None):
-		if brand.is_vaccine:
-			gmGuiHelpers.gm_show_info (
-				aTitle = _('Editing medication'),
-				aMessage = _(
-					'Cannot edit the medication\n'
-					'\n'
-					' "%s" (%s)\n'
-					'\n'
-					'because it is a vaccine. Please edit it\n'
-					'from the vaccine management section !\n'
-				) % (brand['brand'], brand['preparation'])
-			)
-			return False
+	def edit(brand):
+		if brand is not None:
+			if brand.is_vaccine:
+				gmGuiHelpers.gm_show_info (
+					aTitle = _('Editing medication'),
+					aMessage = _(
+						'Cannot edit the medication\n'
+						'\n'
+						' "%s" (%s)\n'
+						'\n'
+						'because it is a vaccine. Please edit it\n'
+						'from the vaccine management section !\n'
+					) % (brand['brand'], brand['preparation'])
+				)
+				return False
 
 		return edit_branded_drug(parent = parent, branded_drug = brand, single_entry = True)
 	#------------------------------------------------------------
@@ -873,7 +875,7 @@ def manage_branded_drugs(parent=None, ignore_OK_button=False):
 #------------------------------------------------------------
 def edit_branded_drug(parent=None, branded_drug=None, single_entry=False):
 	if branded_drug is not None:
-		if branded_drug['is_in_use']:
+		if branded_drug.is_in_use_by_patients:
 			gmGuiHelpers.gm_show_info (
 				aTitle = _('Editing drug'),
 				aMessage = _(
@@ -929,7 +931,7 @@ class cBrandedDrugEAPnl(wxgBrandedDrugEAPnl.wxgBrandedDrugEAPnl, gmEditArea.cGen
 	def _valid_for_save(self):
 
 		if self.data is not None:
-			if self.data['is_in_use']:
+			if self.data.is_in_use_by_patients:
 				gmDispatcher.send(signal = 'statustext', msg = _('Cannot edit drug brand. It is in use.'), beep = True)
 				return False
 
@@ -947,6 +949,24 @@ class cBrandedDrugEAPnl(wxgBrandedDrugEAPnl.wxgBrandedDrugEAPnl, gmEditArea.cGen
 		else:
 			self._PRW_preparation.display_as_valid(True)
 
+		if validity is True:
+			if len(self.__component_substances) == 0:
+				wants_empty = gmGuiHelpers.gm_show_question (
+					title = _('Checking brand data'),
+					question = _(
+						'You have not selected any substances\n'
+						'as drug components.\n'
+						'\n'
+						'Without components you will not be able to\n'
+						'use this drug for documenting patient care.\n'
+						'\n'
+						'Are you sure you want to save\n'
+						'it without components ?'
+					)
+				)
+				if not wants_empty:
+					validity = False
+
 		if validity is False:
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save branded drug. Invalid or missing essential input.'))
 
@@ -1194,6 +1214,16 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 		has_component = (self._PRW_component.GetData() is not None)
 		has_substance = (self._PRW_substance.GetValue().strip() != u'')
 
+		self._PRW_component.display_as_valid(True)
+
+		# cannot add duplicate components
+		if self.mode == 'new':
+			if has_component:
+				emr = gmPerson.gmCurrentPatient().get_emr()
+				if emr.substance_intake_exists(pk_component = self._PRW_component.GetData()):
+					self._PRW_component.display_as_valid(False)
+					validity = False
+
 		# must have either brand or substance
 		if (has_component is False) and (has_substance is False):
 			self._PRW_substance.display_as_valid(False)
@@ -1201,7 +1231,6 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			validity = False
 		else:
 			self._PRW_substance.display_as_valid(True)
-			self._PRW_component.display_as_valid(True)
 
 		# brands already have a preparation, so only required for substances
 		if not has_component:
@@ -1219,33 +1248,33 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			else:
 				self._PRW_episode.display_as_valid(True)
 
-#		# huh ?
-#		if self._CHBOX_approved.IsChecked() is True:
-#			self._PRW_duration.display_as_valid(True)
-#		else:
-#			if self._PRW_duration.GetValue().strip() in [u'', gmTools.u_infinity]:
-#				self._PRW_duration.display_as_valid(True)
-#			else:
-#				if gmDateTime.str2interval(self._PRW_duration.GetValue()) is None:
-#					self._PRW_duration.display_as_valid(False)
-#					validity = False
-#				else:
-#					self._PRW_duration.display_as_valid(True)
+		if self._PRW_duration.GetValue().strip() in [u'', gmTools.u_infinity]:
+			self._PRW_duration.display_as_valid(True)
+		else:
+			if gmDateTime.str2interval(self._PRW_duration.GetValue()) is None:
+				self._PRW_duration.display_as_valid(False)
+				validity = False
+			else:
+				self._PRW_duration.display_as_valid(True)
 
-		# end must be > start if at all
+		# end must be > start but < now, if at all
 		end = self._DP_discontinued.GetData()
 		if end is not None:
-			start = self._DP_started.GetData()
-			if start > end:
-				self._DP_started.display_as_valid(False)
+			if end > gmDateTime.pydt_now_here():
 				self._DP_discontinued.display_as_valid(False)
 				validity = False
 			else:
-				self._DP_started.display_as_valid(True)
-				self._DP_discontinued.display_as_valid(True)
+				start = self._DP_started.GetData()
+				if start > end:
+					self._DP_started.display_as_valid(False)
+					self._DP_discontinued.display_as_valid(False)
+					validity = False
+				else:
+					self._DP_started.display_as_valid(True)
+					self._DP_discontinued.display_as_valid(True)
 
 		if validity is False:
-			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save substance intake. Invalid or missing essential input.'))
+			gmDispatcher.send(signal = 'statustext', msg = _('Input incomplete/invalid for saving as substance intake.'))
 
 		return validity
 	#----------------------------------------------------------------
@@ -1268,7 +1297,7 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			)
 
 		if intake is None:
-			gmDispatcher.send('statustext', msg = _('Cannot add duplicate substance intake.'), beep = True)
+			gmDispatcher.send('statustext', msg = _('Cannot add duplicate of (maybe inactive) substance intake.'), beep = True)
 			return False
 
 		intake['started'] = self._DP_started.GetData()
@@ -1324,6 +1353,7 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 	def _refresh_as_new(self):
 		self._PRW_component.SetText(u'', None)
 		self._TCTRL_brand_ingredients.SetValue(u'')
+		self._TCTRL_brand_ingredients.SetToolTipString(u'')
 
 		self._PRW_substance.SetText(u'', None)
 		self._PRW_substance.Enable(True)
@@ -1351,6 +1381,7 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 	def _refresh_from_existing(self):
 
 		self._TCTRL_brand_ingredients.SetValue(u'')
+		self._TCTRL_brand_ingredients.SetToolTipString(u'')
 
 		if self.data['pk_brand'] is None:
 			self.__refresh_from_existing_substance()
@@ -1411,7 +1442,12 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 
 		brand = gmMedication.cBrandedDrug(aPK_obj = self.data['pk_brand'])
 		if brand['components'] is not None:
-			self._TCTRL_brand_ingredients.SetValue(u'%s\n- %s' % (self.data['brand'], u'\n- '.join(brand['components'])))
+			self._TCTRL_brand_ingredients.SetValue(u'; '.join(brand['components']))
+			tt = u'%s:\n\n- %s' % (
+				self.data['brand'],
+				u'\n- '.join(brand['components'])
+			)
+			self._TCTRL_brand_ingredients.SetToolTipString(tt)
 
 		self._LBL_or.Enable(False)
 		self._LBL_substance.Enable(False)
@@ -1436,6 +1472,7 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			self._PRW_preparation.Enable(True)
 			self._PRW_preparation.SetText(u'', None)
 			self._TCTRL_brand_ingredients.SetValue(u'')
+			self._TCTRL_brand_ingredients.SetToolTipString(u'')
 		else:
 			self._LBL_or.Enable(False)
 			self._LBL_substance.Enable(False)
@@ -1448,7 +1485,12 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			self._PRW_preparation.SetText(comp['preparation'], comp['preparation'])
 			brand = comp.containing_drug
 			if brand['components'] is not None:
-				self._TCTRL_brand_ingredients.SetValue(u'%s\n- %s' % (brand['brand'], u'\n- '.join(brand['components'])))
+				self._TCTRL_brand_ingredients.SetValue(u'; '.join(brand['components']))
+				tt = u'%s:\n\n- %s' % (
+					brand['brand'],
+					u'\n- '.join(brand['components'])
+				)
+				self._TCTRL_brand_ingredients.SetToolTipString(tt)
 	#----------------------------------------------------------------
 	def _on_leave_substance(self):
 		if self._PRW_substance.GetData() is None:
@@ -1465,6 +1507,7 @@ class cSubstanceIntakeEAPnl(wxgCurrentMedicationEAPnl.wxgCurrentMedicationEAPnl,
 			self._LBL_preparation.Enable(True)
 			self._PRW_preparation.Enable(True)
 			self._TCTRL_brand_ingredients.SetValue(u'')
+			self._TCTRL_brand_ingredients.SetToolTipString(u'')
 	#----------------------------------------------------------------
 	def _on_discontinued_date_changed(self, event):
 		if self._DP_discontinued.GetData() is None:
@@ -1577,7 +1620,7 @@ def delete_substance_intake(parent=None, substance=None):
 def edit_intake_of_substance(parent = None, substance=None):
 	ea = cSubstanceIntakeEAPnl(parent = parent, id = -1, substance = substance)
 	dlg = gmEditArea.cGenericEditAreaDlg2(parent = parent, id = -1, edit_area = ea, single_entry = (substance is not None))
-	dlg.SetTitle(gmTools.coalesce(substance, _('Adding substance intake'), _('Editing substance intake')))
+	dlg.SetTitle(gmTools.coalesce(substance, _('Adding medication/non-medication substance intake'), _('Editing medication/non-medication substance intake')))
 	dlg.left_extra_button = (
 		_('Allergy'),
 		_('Document an allergy against this substance.'),
@@ -1681,7 +1724,7 @@ def print_medication_list(parent=None):
 		return False
 
 	# 3) print template
-	printed = gmPrinting.print_file_by_shellscript(filename = pdf_name, jobtype = 'medication_list')
+	printed = gmPrinting.print_files(filenames = [pdf_name], jobtype = 'medication_list')
 	if not printed:
 		gmGuiHelpers.gm_show_error (
 			aMessage = _('Error printing the medication list.'),
@@ -1776,7 +1819,7 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 		self.__prev_cell_0 = None
 		self.__grouping_mode = u'episode'
 		self.__filter_show_unapproved = True
-		self.__filter_show_inactive = False
+		self.__filter_show_inactive = True
 
 		self.__grouping2col_labels = {
 			u'episode': [
@@ -1786,7 +1829,8 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 				_('Schedule'),
 				_('Started'),
 				_('Duration / Until'),
-				_('Brand')
+				_('Brand'),
+				_('Advice')
 			],
 			u'brand': [
 				_('Brand'),
@@ -1795,7 +1839,8 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 				_('Strength'),
 				_('Started'),
 				_('Duration / Until'),
-				_('Episode')
+				_('Episode'),
+				_('Advice')
 			]
 		}
 
@@ -1991,6 +2036,9 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 			else:
 				raise ValueError('unknown grouping mode [%s]' % self.__grouping_mode)
 
+			if med['notes'] is not None:
+				self.SetCellValue(row_idx, 7, gmTools.wrap(text = med['notes'], width = 50))
+
 			if self.__filter_show_unapproved:
 				self.SetCellValue (
 					row_idx,
@@ -2036,20 +2084,12 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 			drug_db.show_info_on_drug(substance_intake = intake)
 	#------------------------------------------------------------
 	def show_renal_insufficiency_info(self):
-
-		if len(self.__row_data) == 0:
-			return
-
-		sel_rows = self.get_selected_rows()
-
-		if len(sel_rows) != 1:
-			return
-
-		webbrowser.open (
-			url = gmMedication.drug2renal_insufficiency_url(search_term = self.get_selected_data()[0]),
-			new = False,
-			autoraise = True
-		)
+		search_term = None
+		if len(self.__row_data) > 0:
+			sel_rows = self.get_selected_rows()
+			if len(sel_rows) == 1:
+				search_term = self.get_selected_data()[0]
+		gmNetworkTools.open_url_in_browser(url = gmMedication.drug2renal_insufficiency_url(search_term = search_term))
 	#------------------------------------------------------------
 	def report_ADR(self):
 
@@ -2061,8 +2101,7 @@ class cCurrentSubstancesGrid(wx.grid.Grid):
 			bias = u'user',
 			default = u'https://dcgma.org/uaw/meldung.php'		# http://www.akdae.de/Arzneimittelsicherheit/UAW-Meldung/UAW-Meldung-online.html
 		)
-
-		webbrowser.open(url = url, new = False, autoraise = True)
+		gmNetworkTools.open_url_in_browser(url = url)
 	#------------------------------------------------------------
 	def prescribe(self):
 		drug_db = get_drug_database()
